from __future__ import print_function
from __future__ import absolute_import
import sys
import time
import os
import math
import numpy
import inspect
import wx
import wx.lib.agw.ultimatelistctrl as ULC
from . import subUtils.xrc.base as xrcbase
from . import subUtils.execCommandCommon as ecc
import utsusemi.DNA.ana.Reduction.SearchPathDNA as SPD
#-------------------------------------------------------------------------------------------------
#AGW(http://xoomer.virgilio.it/infinity77/AGW_Docs/), ver > 0.9.1 ?
#-------------------------------------------------------------------------------------------------

NUMBER_OF_ROW        = 50

DEFAULT_DATA_PATH    = os.path.join(os.path.dirname(os.path.abspath(__file__)))

CTRL_COLUMN_WIDTHS   = [30,90,90,120,130,90,90]
CTRL_COLUMN_LABELS   = [" ",
                        "      RunNo",
                        "TOF offset",
                        "         Slit",
                        "        TrigNet",
                        " Start Time ",
                        "  End Time "]

NUM_OF_CONTROL       = len(CTRL_COLUMN_LABELS)

CTRL_TABLE_SIZE      = [sum(CTRL_COLUMN_WIDTHS)+240,400]

THERMOMETER_TYPE     = ["A","B","C1","C2","C3","C4","D1","D2","D3","D4"]

DEFAULT_SETTING_PATH = "default.csv"

#-------------------------------------------------------------------------------------------------
def _isFloatStr(val):
    try:
        ret=float(val)
        return True
    except:
        return False
#-------------------------------------------------------------------------------------------------
def _isIntStr(val):
    try:
        ret=int(val)
        return True
    except:
        return False
#-------------------------------------------------------------------------------------------------
class MainPanel(xrcbase.PanelBase):
    def __init__(self, parent,bgcolour=(0,0,255)):
        xrcbase.PanelBase.__init__(self, parent, 'dataReduction.xrc', 'mainPanel')

        self.parent = parent

        ctrlPanel = self.getControl('ctrl panel')
        self.ctrlPanelObj = CtrlPanel(ctrlPanel)
        ctrlSizer = wx.BoxSizer(wx.VERTICAL)
        ctrlSizer.Add(self.ctrlPanelObj, flag = wx.EXPAND, proportion=1)
        ctrlPanel.SetSizer(ctrlSizer)
        ctrlPanel.SetMinSize(CTRL_TABLE_SIZE)

        objnames = ["start button", "clear button", "quit button","binInfo load button",
                    "maskInfo load button", "effInfo load button", "setting load button",
                    "setting save button", "read temp button", "etRangeNormRB", "etRangeUserRB"]
        evtType  = [wx.EVT_BUTTON, wx.EVT_BUTTON, wx.EVT_BUTTON, wx.EVT_BUTTON, wx.EVT_BUTTON, wx.EVT_BUTTON,
                    wx.EVT_BUTTON, wx.EVT_BUTTON, wx.EVT_BUTTON, wx.EVT_RADIOBUTTON, wx.EVT_RADIOBUTTON]
        funcs    = [self.start, self.clear, self.close, self.popupFileDialog, self.popupFileDialog,
                    self.popupFileDialog, self.popupFileDialog, self.popupFileDialog, self.onReadTemp,
                    self.selectETobject,self.selectETobject]

        for name,evt,func in zip(objnames,evtType,funcs):
            self.getControl(name).Bind(evt,func)

        self.selectETobject(None)
        self.getControl("thermometer Type").SetItems(THERMOMETER_TYPE)
        self.getControl("thermometer Type").SetSelection(0)

        self.initialize()
#-------------------------------------------------------------------------------------------------
    def _getValueFromList(self,listVal,default,*args):
        depth = len(args)
        try:
            if depth==1:
                val=listVal[args[0]]
            elif depth==2:
                val=listVal[args[0]][args[1]]
            elif depth==3:
                val=listVal[args[0]][args[1]][args[2]]
            elif depth==4:
                val=listVal[args[0]][args[1]][args[2]][args[3]]
            else:
                val=default
        except:
            val=default

        return val
#-------------------------------------------------------------------------------------------------
    def initialize(self):
        self.loadSetting(DEFAULT_SETTING_PATH)
        pass
#-------------------------------------------------------------------------------------------------
    def selectETobject(self, event):
        flag = self.getControl("etRangeNormRB").GetValue()
        objnames = ["binInfo label", "binInfo Path", "binInfo load button"]
        for name in objnames:
            self.getControl(name).Enable(not flag)

        objnames = ["Emin label", "Emin control", "Emax label", "Emax control",
                    "Ebin label", "Ebin control", "eunit label"]
        for name in objnames:
            self.getControl(name).Enable(flag)
#-------------------------------------------------------------------------------------------------
    def popupFileDialog(self, event):

        if type(event.GetEventObject())==wx._controls.Button:
            buttonName = event.GetEventObject().GetName()
        elif type(event.GetEventObject())==wx._core.Menu:
            buttonName = event.GetEventObject().GetLabel(event.GetId())

        if buttonName == "setting save button":
            title = 'Save File Specify'
            dialogType = wx.SAVE
            fileDialog = wx.FileDialog(self, title,
                                             style = dialogType,
                                             defaultDir = DEFAULT_DATA_PATH)
            #wildcard = 'Text file (*.txt)|*.txt'
            #fileDialog = wx.FileDialog(self, title,
            #                                 style = dialogType,
            #                                 wildcard = wildcard)

        else:
            title = 'Open File Specify('+buttonName+')'
            dialogType = wx.FD_OPEN

            if buttonName=="maskInfo load button":
                #defaultDataDir = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml")
                ret = SPD.CheckForOutput( "xml" )
                if ret=="":
                    raise UserWarning( "MultiDataCobine/MainFrame::popupFileDialog >> Not found xml directory." )
                else:
                    defaultDataDir = ret
            elif buttonName=="effInfo load button":
                #defaultDataDir = os.path.join(os.environ["HOME"],"ana","setup")
                ret = SPD.CheckForOutput( "setup" )
                if ret=="":
                    raise UserWarning( "MultiDataCobine/MainFrame::popupFileDialog >> Not found setup directory." )
                else:
                    defaultDataDir = ret
            else:
                defaultDataDir = DEFAULT_DATA_PATH

            fileDialog = wx.FileDialog(self, title,
                                             style = dialogType,
                                             defaultDir = defaultDataDir)

            #wildcard = 'Text file (*.txt)|*.txt'
            #fileDialog = wx.FileDialog(self, title,
            #                                 style = dialogType,
            #                                 wildcard = wildcard,
            #                                 defaultDir = default_datapath)
        #--------------------------------------------------------------------------------

        flag = fileDialog.ShowModal();
        if  flag == wx.ID_OK:
            filepath = str(fileDialog.GetPath())
            name=fileDialog.GetFilename()

            if buttonName   == "binInfo load button":
                obj = self.getControl("binInfo Path")
                obj.SetValue(filepath)
                self.loadBinInfo(filepath)
            elif buttonName == "maskInfo load button":
                obj = self.getControl("maskInfo Path")
                obj.SetValue(filepath)
                #self.loadMaskInfo(filepath)
            elif buttonName == "effInfo load button":
                obj = self.getControl("effInfo Path")
                obj.SetValue(filepath)
                #self.loadEfficiencyInfo(filepath)
            elif buttonName == "setting load button":
                self.loadSetting(filepath)
            elif buttonName == "setting save button":
                self.saveSetting(filepath)
            else:
                pass
            time.sleep(0.5)
            pass

        elif flag == wx.ID_CANCEL:
            if buttonName == "maskInfo load button":
                obj = self.getControl("maskInfo Path")
                obj.SetValue("")
            elif buttonName == "effInfo load button":
                obj = self.getControl("effInfo Path")
                obj.SetValue("")

#-------------------------------------------------------------------------------------------------
    def loadBinInfo(self,filepath):
        from . import subUtils.BinInfoReaderBase as BIR
        binsText = BIR._readBinInfo(filepath,"\n",",")
        return binsText
#-------------------------------------------------------------------------------------------------
    def loadMaskInfo(self,filepath):
        pass
#-------------------------------------------------------------------------------------------------
    def loadEfficiencyInfo(self,filepath):
        pass
#-------------------------------------------------------------------------------------------------
    def loadSetting(self,filepath):
        import csv
        self.clear(event=None)

        reader = csv.reader(open(filepath, 'rb'))
        lines = [line for line in reader]

        val = self._getValueFromList(lines,"0",0,0)
        if val=="1":
            self.getControl("etRangeNormRB").SetValue(1)
            self.getControl("etRangeUserRB").SetValue(0)
        else:
            self.getControl("etRangeNormRB").SetValue(0)
            self.getControl("etRangeUserRB").SetValue(1)

        val = self._getValueFromList(lines,"",0,1)
        self.getControl("Emin control").SetValue(val)

        val = self._getValueFromList(lines,"",0,2)
        self.getControl("Emax control").SetValue(val)

        val = self._getValueFromList(lines,"",0,3)
        self.getControl("Ebin control").SetValue(val)

        val = self._getValueFromList(lines,"",1,1)
        self.getControl("binInfo Path").SetValue(val)

        val = self._getValueFromList(lines,"",2,0)
        self.getControl("LambdaCenter control").SetValue(val)

        val = self._getValueFromList(lines,"",2,1)
        self.getControl("deltaLambda control").SetValue(val)

        val = self._getValueFromList(lines,"",3,0)
        self.getControl("Qmin control").SetValue(val)

        val = self._getValueFromList(lines,"",3,1)
        self.getControl("Qmax control").SetValue(val)

        val = self._getValueFromList(lines,"",3,2)
        self.getControl("Qbin control").SetValue(val)

        val = self._getValueFromList(lines,THERMOMETER_TYPE[0],4,0)
        self.getControl("thermometer Type").SetStringSelection(val)

        val = self._getValueFromList(lines,"",5,0)
        self.getControl("f chopper control").SetValue(val)

        val = self._getValueFromList(lines,"",5,1)
        self.getControl("tof source min").SetValue(val)

        val = self._getValueFromList(lines,"",5,2)
        self.getControl("tof source max").SetValue(val)

        val = self._getValueFromList(lines,"",6,0)
        self.getControl("maskInfo Path").SetValue(val)

        val = self._getValueFromList(lines,"",7,0)
        self.getControl("effInfo Path").SetValue(val)

        self.selectETobject(None)

        table_param = lines[9:]
        params = []
        ind=0
        i  =0
        for param in table_param:
            params += param[1:]
            if len(param)>0:
                if param[0]=="1":
                    ind=i
                i+=1
        self.ctrlPanelObj.setValue(params)
        self.ctrlPanelObj.setCheckRunNumber(ind)
#-------------------------------------------------------------------------------------------------
    def saveSetting(self,filepath):
        import csv
        writer = csv.writer(open(filepath, 'wb'))
        # Line 1
        data=[]
        data.append(str(int(self.getControl("etRangeNormRB").GetValue())))
        data.append(str(self.getControl("Emin control").GetValue()))
        data.append(str(self.getControl("Emax control").GetValue()))
        data.append(str(self.getControl("Ebin control").GetValue()))
        writer.writerow(data)
        # Line 2
        data=[]
        data.append(str(int(self.getControl("etRangeUserRB").GetValue())))
        data.append(str(self.getControl("binInfo Path").GetValue()))
        writer.writerow(data)
        # Line 3
        data=[]
        data.append(str(self.getControl("LambdaCenter control").GetValue()))
        data.append(str(self.getControl("deltaLambda control").GetValue()))
        writer.writerow(data)
        # Line 4
        data=[]
        data.append(str(self.getControl("Qmin control").GetValue()))
        data.append(str(self.getControl("Qmax control").GetValue()))
        data.append(str(self.getControl("Qbin control").GetValue()))
        writer.writerow(data)
        # Line 5
        data=[self.getControl("thermometer Type").GetStringSelection()]
        writer.writerow(data)
        # Line 6
        data=[]
        data.append(str(self.getControl("f chopper control").GetValue()))
        data.append(str(self.getControl("tof source min").GetValue()))
        data.append(str(self.getControl("tof source max").GetValue()))
        writer.writerow(data)
        # Line 7
        data=[self.getControl("maskInfo Path").GetValue()]
        writer.writerow(data)
        # Line 8
        data=[self.getControl("effInfo Path").GetValue()]
        writer.writerow(data)
        # Line 9
        writer.writerow([])
        # Line 10
        objs = self.ctrlPanelObj.getValue()

        for i in range(len(objs)):
            if isinstance(objs[i],bool):
                objs[i]=str(int(objs[i]))

        objs = [objs[i:i+(NUM_OF_CONTROL-1)] for i in range(0,len(objs),(NUM_OF_CONTROL-1))]

        runNo = str(self.ctrlPanelObj.getCheckRunNumber())
        for i in range(len(objs)):
            if objs[i][0]==runNo:
                objs[i] = ["1"] + objs[i]
            else:
                objs[i] = ["0"] + objs[i]

        for obj in objs:
            if "" in obj:
                pass
            else:
                writer.writerow(obj)
#-------------------------------------------------------------------------------------------------
    def onReadTemp(self,event):
        print("L :",inspect.currentframe().f_lineno,"/",__file__,"/",\
              inspect.currentframe().f_code.co_name)

        def command(runNo,thermoType):
            try:
                import uGao.MPlot as mp
                import Cmm
                EC = Cmm.CheckSampleTempByRunNumber(runNo,thermoType)
                mp.MPlot(EC)
            except:
                print("import Error / Manyo, MPlot, Cmm")

        runNo      = int(self.ctrlPanelObj.getCheckRunNumber())
        obj        = self.getControl("thermometer Type")
        thermoType = obj.GetStringSelection()

        print("# Run No.", runNo)
        print("# Thermo Type.", thermoType)

        ecc.ExecCommandCommon(command,event,args=(runNo,thermoType),objs=[obj])
#-------------------------------------------------------------------------------------------------
    def checkValues(self,vals,names):
        ret=[]
        for val,name in zip(vals,names):
            flag = not _isFloatStr(val)
            if flag:
                ret.append([val,name])

        if len(ret)==0:
            return True
        else:
            errorMessage=""
            for err in ret:
                if err[0].strip()=="":
                    errorMessage += err[1]+" is empty.\n"
                else:
                    errorMessage += err[1]+" : "+err[0]+" : is invalid.\n"

            dlg = wx.MessageDialog( self, errorMessage,  "Error",  wx.OK)
            dlg.ShowModal()

            return False
#-------------------------------------------------------------------------------------------------
    def checkFilePaths(self,vals,names):
        ret=[]
        for val,name in zip(vals,names):
            flag = not os.path.exists(val) and val.strip()!=""
            if flag:
                ret.append([val,name])

        if len(ret)==0:
            return True
        else:
            errorMessage=""
            for err in ret:
                errorMessage += err[1]+" : "+err[0]+" : is not found.\n"

            dlg = wx.MessageDialog( self, errorMessage,  "Error",  wx.OK)
            dlg.ShowModal()

            return False
#-------------------------------------------------------------------------------------------------
    def start(self,event):

        # Line 1
        etFlag = self.getControl("etRangeNormRB").GetValue()
        eMin = self.getControl("Emin control").GetValue()
        eMax = self.getControl("Emax control").GetValue()
        eBin = self.getControl("Ebin control").GetValue()

        # Line 2
        binInfoPath = self.getControl("binInfo Path").GetValue()

        # Line 3
        LambdaCenter = self.getControl("LambdaCenter control").GetValue()
        deltaLambda  = self.getControl("deltaLambda control").GetValue()

        # Line 4
        qMin = self.getControl("Qmin control").GetValue()
        qMax = self.getControl("Qmax control").GetValue()
        qBin = self.getControl("Qbin control").GetValue()

        # Line 6
        fChop = self.getControl("f chopper control").GetValue()
        tofSrcMin = self.getControl("tof source min").GetValue()
        tofSrcMax = self.getControl("tof source max").GetValue()

        # Line 7
        maskInfoPath = self.getControl("maskInfo Path").GetValue()
        # Line 8
        effInfoPath  = self.getControl("effInfo Path").GetValue()

        # Line 10
        objs = self.ctrlPanelObj.getValue()
        for i in range(len(objs)):
            if isinstance(objs[i],bool):
                objs[i]=str(int(objs[i]))

        objs = [objs[i:i+(NUM_OF_CONTROL-1)] for i in range(0,len(objs),(NUM_OF_CONTROL-1))]
        i=0
        j=0
        index=[]
        while i<len(objs):
            if "" in objs[i] or u'' in objs[i]:
                del objs[i]
            else:
                index.append(j)
                i+=1
            j+=1

        slitwidths  = [int(val[2]) for val in objs] # if val==1:3cm, else if val==0:1cm.
        useTrignets = [int(val[3]) for val in objs] # if val==1:Use, else if val==0:NotUse.

        timeRanges  = [val[4]+","+val[5] for val in objs]

        if etFlag:
            hwParam     = eBin + "," + eMin + "," + eMax
        else:
            hwParam     = self.loadBinInfo(binInfoPath)

        lambdaParam = LambdaCenter + "," + deltaLambda

        tofSource   = tofSrcMin + "," + tofSrcMax

        # Error Check
        vals  = [eMin,eMax,eBin,LambdaCenter,deltaLambda,qMin,qMax,qBin,fChop,tofSrcMin,tofSrcMax]
        names = ["emin","emax","ebin","LambdaCenter","deltaLambda","qmin","qmax","qbin","f chopper",
                 "tof source start","tof source end"]
        for j,obj in zip(index,objs):
            runNo     = obj[0]
            tofOffset = obj[1]
            startTime = obj[4]
            endTime   = obj[5]
            js = str(j+1)
            vals  += [runNo,tofOffset,startTime,endTime]
            names += ["runNo"+js,"tofOffset"+js,"startTime"+js,"endTime"+js]
        ret1 = self.checkValues(vals,names)

        if etFlag:
            vals  = [maskInfoPath,effInfoPath]
            names = ["mask info path","efficiency info path"]
        else:
            vals  = [binInfoPath,maskInfoPath,effInfoPath]
            names = ["bin info path","mask info path","efficiency info path"]
        ret2 = self.checkFilePaths(vals,names)

        if not (ret1 and ret2):
            pass
        else:
            startQ = float(qMin)
            endQ   = float(qMax)
            deltaQ = float(qBin)
            fChop  = float(fChop)
            runNos = [int(val[0])   for val in objs]
            offsets= [float(val[1]) for val in objs]
            from . import command
            ecc.ExecCommandCommon(command.command,event,
                                  args=(runNos, offsets, timeRanges, hwParam, lambdaParam,
                                        tofSource, startQ, endQ, deltaQ, slitwidths, useTrignets,fChop,
                                        maskInfoPath, effInfoPath),
                                  objs=[self])
            pass
#-------------------------------------------------------------------------------------------------
    def clear(self,event):

        self.getControl("etRangeNormRB").SetValue(1)
        self.getControl("etRangeUserRB").SetValue(0)

        objnames =["Emin control", "Emax control", "Ebin control", "binInfo Path",
                   "LambdaCenter control", "deltaLambda control", "Qmin control",
                   "Qmax control", "Qbin control", "f chopper control", "tof source min",
                   "tof source max", "maskInfo Path", "effInfo Path"]
        for name in objnames:
            self.getControl(name).SetValue("")

        self.getControl("thermometer Type").SetStringSelection(THERMOMETER_TYPE[0])

        self.ctrlPanelObj.clear()
#-------------------------------------------------------------------------------------------------
    def close(self,event):
        self.parent.Destroy()
        pass
#-------------------------------------------------------------------------------------------------
class CtrlPanel(wx.Panel):
    def __init__(self,parent):
        wx.Panel.__init__(self, parent)

        scroll = wx.ScrolledWindow(self, -1)
        scroll.SetScrollRate(1,1)

        self.ctrlObjs=[]

        self.checkBoxObjs=[]

        ulist = ULC.UltimateListCtrl(scroll, wx.ID_ANY,
                                     agwStyle=wx.LC_REPORT|\
                                              wx.LC_VRULES|\
                                              wx.LC_HRULES|\
                                              wx.LC_SINGLE_SEL|\
                                              ULC.ULC_HAS_VARIABLE_ROW_HEIGHT)

        for i,label in enumerate(CTRL_COLUMN_LABELS):
            ulist.InsertColumn(i, label)

        for i,width in enumerate(CTRL_COLUMN_WIDTHS):
            ulist.SetColumnWidth(i,width)

        for i in range(NUMBER_OF_ROW):
            index = ulist.InsertStringItem(sys.maxsize, "")
            checkbox_1 = wx.CheckBox(ulist,wx.ID_ANY,"")
            ulist.SetItemWindow(index, 0, checkbox_1)
            self.checkBoxObjs.append(checkbox_1)
            checkbox_1.Bind(wx.EVT_CHECKBOX,self.onCheckRunNumber)

            text_1 = wx.TextCtrl(ulist, wx.ID_ANY, style = wx.TE_RIGHT)
            ulist.SetItemWindow(index, 1, text_1)
            self.ctrlObjs.append(text_1)

            text_2 = wx.TextCtrl(ulist, wx.ID_ANY, style = wx.TE_RIGHT)
            ulist.SetItemWindow(index, 2, text_2)
            self.ctrlObjs.append(text_2)

            radio_panel1,btn_obj1 = self.getRadioBoxPanel(ulist,wx.ID_ANY,"3cm","1cm")
            ulist.SetItemWindow(index, 3, radio_panel1)
            self.ctrlObjs.append(btn_obj1)

            radio_panel2,btn_obj2 = self.getRadioBoxPanel(ulist,wx.ID_ANY,"Use","NotUse")
            ulist.SetItemWindow(index, 4, radio_panel2)
            self.ctrlObjs.append(btn_obj2)

            text_3 = wx.TextCtrl(ulist, wx.ID_ANY, style = wx.TE_RIGHT)
            ulist.SetItemWindow(index, 5, text_3)
            self.ctrlObjs.append(text_3)

            text_4 = wx.TextCtrl(ulist, wx.ID_ANY, style = wx.TE_RIGHT)
            ulist.SetItemWindow(index, 6, text_4)
            self.ctrlObjs.append(text_4)

        self.checkBoxObjs[0].SetValue(True)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(ulist, 1, wx.EXPAND)
        scroll.SetSizer(sizer)

        sizerf = wx.BoxSizer(wx.VERTICAL)
        sizerf.Add(scroll, 1, wx.EXPAND)
        self.SetSizer(sizerf)
#-------------------------------------------------------------------------------------------------
    def onCheckRunNumber(self,event):
        obj=event.GetEventObject()
        if obj.GetValue():
            for cb in self.checkBoxObjs:
                cb.SetValue(False)
        else:
            pass
        obj.SetValue(True)
#-------------------------------------------------------------------------------------------------
    def getCheckRunNumber(self):
        for i,cb in enumerate(self.checkBoxObjs):
            if cb.GetValue():
                return self.ctrlObjs[i*(NUM_OF_CONTROL-1)].GetValue()
#-------------------------------------------------------------------------------------------------
    def setCheckRunNumber(self,val):
        for i,cb in enumerate(self.checkBoxObjs):
            if i==val:
                cb.SetValue(1)
            else:
                cb.SetValue(0)
#-------------------------------------------------------------------------------------------------
    def getValue(self):
        ret=[obj.GetValue() for obj in self.ctrlObjs]
        return ret
#-------------------------------------------------------------------------------------------------
    def setValue(self,vals):
        for i,val in enumerate(vals):
            if i<len(self.ctrlObjs):
                if isinstance(self.ctrlObjs[i], wx._controls.TextCtrl):
                    self.ctrlObjs[i].SetValue(val)
                elif isinstance(self.ctrlObjs[i], wx._controls.RadioButton):
                    val = int(val)
                    objs = [ item.GetWindow() for item in self.ctrlObjs[i].GetParent().GetSizer().GetChildren() ]
                    if val==1:
                        objs[0].SetValue(True)
                    else:
                        objs[1].SetValue(True)
                    #self.ctrlObjs[i].SetValue(val)
                else:
                    pass
#-------------------------------------------------------------------------------------------------
    def clear(self):
        for i,cb in enumerate(self.checkBoxObjs):
            if i==0:
                cb.SetValue(True)
            else:
                cb.SetValue(False)

        for i,obj in enumerate(self.ctrlObjs):
            if i%(NUM_OF_CONTROL-1)==2 or i%(NUM_OF_CONTROL-1)==3:
                obj.SetValue(True)
            else:
                obj.SetValue("")
#-------------------------------------------------------------------------------------------------
    def getRadioBoxPanel(self,parent,ids,choice1,choice2):
        radio_panel = wx.Panel(parent,ids)
        radio_panel.SetBackgroundColour("#FFFFFF")
        radio_sizer = wx.BoxSizer(wx.HORIZONTAL)
        radiobutton_1 = wx.RadioButton(radio_panel,wx.ID_ANY,choice1)
        radiobutton_2 = wx.RadioButton(radio_panel,wx.ID_ANY,choice2)
        radio_sizer.Add(radiobutton_1)
        radio_sizer.Add(radiobutton_2)
        radio_panel.SetSizer(radio_sizer)
        radio_panel.Fit()
        return radio_panel,radiobutton_1
#-------------------------------------------------------------------------------------------------
class MainFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, wx.ID_ANY, 'Data Reduction DNA', 
                          style = wx.DEFAULT_FRAME_STYLE)

        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        self.panel = MainPanel(self)
        sizer.Add(self.panel, flag = wx.ALL | wx.EXPAND, border = 5, proportion = 1)

        menuBar = MainMenuBar()
        self.SetMenuBar(menuBar)

        self.Bind(wx.EVT_MENU, self.onAbout, menuBar.menuItemAbout)
        self.Bind(wx.EVT_MENU, self.onExit , menuBar.menuItemExit )

        self.CreateStatusBar()
        self.Fit()
#-------------------------------------------------------------------------------------------------
    def onAbout(self, event):
        description = ""
        licence = "LICENCE"
        info = wx.AboutDialogInfo()
        info.SetName('Data Reduction DNA')
        info.SetVersion('0.0.0')
        info.SetDescription(description)
        info.SetCopyright('2014/xx/xx')
        info.SetWebSite('http://www/')
        #info.SetLicence(licence)
        info.AddDeveloper('')
        #info.AddDocWriter('')
        #info.AddArtist('')
        #info.AddTranslator('')
        wx.AboutBox(info)
#-------------------------------------------------------------------------------------------------
    def onExit(self, event):
        self.Destroy()
#-------------------------------------------------------------------------------------------------
class MainMenuBar(wx.MenuBar):
    def __init__(self):
        wx.MenuBar.__init__(self)

        menuFile = wx.Menu()
        self.menuItemExit               = menuFile.Append(1,'   Exit')

        menuHelp = wx.Menu()
        self.menuItemAbout              = menuHelp.Append(2,'   About')

        self.Append(menuFile,'&File')
        self.Append(menuHelp,'&Help')
#-------------------------------------------------------------------------------------------------
app = wx.PySimpleApp()

frame = MainFrame()
app.SetTopWindow(frame)
frame.Show()

app.MainLoop()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'D4mat2SlicerQ_Main.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QGroupBox,
    QLabel, QLineEdit, QMainWindow, QMenuBar,
    QPushButton, QSizePolicy, QStatusBar, QToolButton,
    QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(582, 790)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.Panel1 = QGroupBox(self.centralwidget)
        self.Panel1.setObjectName(u"Panel1")
        self.Panel1.setGeometry(QRect(10, 220, 560, 161))
        self.txt14_p1 = QLineEdit(self.Panel1)
        self.txt14_p1.setObjectName(u"txt14_p1")
        self.txt14_p1.setGeometry(QRect(310, 130, 60, 20))
        self.txt13_p1 = QLineEdit(self.Panel1)
        self.txt13_p1.setObjectName(u"txt13_p1")
        self.txt13_p1.setGeometry(QRect(150, 130, 60, 20))
        self.label15 = QLabel(self.Panel1)
        self.label15.setObjectName(u"label15")
        self.label15.setGeometry(QRect(0, 130, 68, 17))
        self.txt2_p1 = QLineEdit(self.Panel1)
        self.txt2_p1.setObjectName(u"txt2_p1")
        self.txt2_p1.setGeometry(QRect(260, 30, 60, 20))
        self.label7 = QLabel(self.Panel1)
        self.label7.setObjectName(u"label7")
        self.label7.setGeometry(QRect(190, 50, 65, 20))
        self.txt11_p1 = QLineEdit(self.Panel1)
        self.txt11_p1.setObjectName(u"txt11_p1")
        self.txt11_p1.setGeometry(QRect(260, 100, 60, 20))
        self.txt15_p1 = QLineEdit(self.Panel1)
        self.txt15_p1.setObjectName(u"txt15_p1")
        self.txt15_p1.setGeometry(QRect(460, 130, 60, 20))
        self.txt5_p1 = QLineEdit(self.Panel1)
        self.txt5_p1.setObjectName(u"txt5_p1")
        self.txt5_p1.setGeometry(QRect(260, 50, 60, 20))
        self.txt6_p1 = QLineEdit(self.Panel1)
        self.txt6_p1.setObjectName(u"txt6_p1")
        self.txt6_p1.setGeometry(QRect(410, 50, 60, 20))
        self.label4 = QLabel(self.Panel1)
        self.label4.setObjectName(u"label4")
        self.label4.setGeometry(QRect(190, 30, 65, 20))
        self.label6 = QLabel(self.Panel1)
        self.label6.setObjectName(u"label6")
        self.label6.setGeometry(QRect(30, 50, 65, 20))
        self.label14 = QLabel(self.Panel1)
        self.label14.setObjectName(u"label14")
        self.label14.setGeometry(QRect(340, 100, 65, 20))
        self.label18 = QLabel(self.Panel1)
        self.label18.setObjectName(u"label18")
        self.label18.setGeometry(QRect(380, 130, 31, 17))
        self.txt4_p1 = QLineEdit(self.Panel1)
        self.txt4_p1.setObjectName(u"txt4_p1")
        self.txt4_p1.setGeometry(QRect(100, 50, 60, 20))
        self.txt8_p1 = QLineEdit(self.Panel1)
        self.txt8_p1.setObjectName(u"txt8_p1")
        self.txt8_p1.setGeometry(QRect(260, 80, 60, 20))
        self.label8 = QLabel(self.Panel1)
        self.label8.setObjectName(u"label8")
        self.label8.setGeometry(QRect(340, 50, 65, 20))
        self.chRotateAxis3rd = QComboBox(self.Panel1)
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.setObjectName(u"chRotateAxis3rd")
        self.chRotateAxis3rd.setGeometry(QRect(410, 130, 50, 25))
        font = QFont()
        font.setPointSize(10)
        self.chRotateAxis3rd.setFont(font)
        self.chRotateAxis3rd.setEditable(False)
        self.label17 = QLabel(self.Panel1)
        self.label17.setObjectName(u"label17")
        self.label17.setGeometry(QRect(230, 130, 31, 17))
        self.txt1_p1 = QLineEdit(self.Panel1)
        self.txt1_p1.setObjectName(u"txt1_p1")
        self.txt1_p1.setGeometry(QRect(100, 30, 60, 20))
        self.txt9_p1 = QLineEdit(self.Panel1)
        self.txt9_p1.setObjectName(u"txt9_p1")
        self.txt9_p1.setGeometry(QRect(410, 80, 60, 20))
        self.txt12_p1 = QLineEdit(self.Panel1)
        self.txt12_p1.setObjectName(u"txt12_p1")
        self.txt12_p1.setGeometry(QRect(410, 100, 60, 20))
        self.label11 = QLabel(self.Panel1)
        self.label11.setObjectName(u"label11")
        self.label11.setGeometry(QRect(340, 80, 65, 20))
        self.label10 = QLabel(self.Panel1)
        self.label10.setObjectName(u"label10")
        self.label10.setGeometry(QRect(190, 80, 65, 20))
        self.chRotateAxis2nd = QComboBox(self.Panel1)
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.setObjectName(u"chRotateAxis2nd")
        self.chRotateAxis2nd.setGeometry(QRect(260, 130, 50, 25))
        self.chRotateAxis2nd.setFont(font)
        self.chRotateAxis2nd.setEditable(False)
        self.label13 = QLabel(self.Panel1)
        self.label13.setObjectName(u"label13")
        self.label13.setGeometry(QRect(190, 100, 65, 20))
        self.chRotateAxis1st = QComboBox(self.Panel1)
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.setObjectName(u"chRotateAxis1st")
        self.chRotateAxis1st.setGeometry(QRect(100, 130, 50, 25))
        self.chRotateAxis1st.setFont(font)
        self.chRotateAxis1st.setEditable(False)
        self.label3 = QLabel(self.Panel1)
        self.label3.setObjectName(u"label3")
        self.label3.setGeometry(QRect(30, 30, 65, 20))
        self.txt3_p1 = QLineEdit(self.Panel1)
        self.txt3_p1.setObjectName(u"txt3_p1")
        self.txt3_p1.setGeometry(QRect(410, 30, 60, 20))
        self.label16 = QLabel(self.Panel1)
        self.label16.setObjectName(u"label16")
        self.label16.setGeometry(QRect(70, 130, 31, 17))
        self.txt7_p1 = QLineEdit(self.Panel1)
        self.txt7_p1.setObjectName(u"txt7_p1")
        self.txt7_p1.setGeometry(QRect(100, 80, 60, 20))
        self.label5 = QLabel(self.Panel1)
        self.label5.setObjectName(u"label5")
        self.label5.setGeometry(QRect(340, 30, 65, 20))
        self.txt10_p1 = QLineEdit(self.Panel1)
        self.txt10_p1.setObjectName(u"txt10_p1")
        self.txt10_p1.setGeometry(QRect(100, 100, 60, 20))
        self.label9 = QLabel(self.Panel1)
        self.label9.setObjectName(u"label9")
        self.label9.setGeometry(QRect(30, 80, 65, 20))
        self.label12 = QLabel(self.Panel1)
        self.label12.setObjectName(u"label12")
        self.label12.setGeometry(QRect(30, 100, 65, 20))
        self.Panel2 = QGroupBox(self.centralwidget)
        self.Panel2.setObjectName(u"Panel2")
        self.Panel2.setGeometry(QRect(10, 410, 560, 131))
        self.txt1_p2 = QLineEdit(self.Panel2)
        self.txt1_p2.setObjectName(u"txt1_p2")
        self.txt1_p2.setGeometry(QRect(40, 40, 55, 20))
        self.label25 = QLabel(self.Panel2)
        self.label25.setObjectName(u"label25")
        self.label25.setGeometry(QRect(10, 40, 31, 17))
        self.txt2_p2 = QLineEdit(self.Panel2)
        self.txt2_p2.setObjectName(u"txt2_p2")
        self.txt2_p2.setGeometry(QRect(100, 40, 55, 20))
        self.txt3_p2 = QLineEdit(self.Panel2)
        self.txt3_p2.setObjectName(u"txt3_p2")
        self.txt3_p2.setGeometry(QRect(160, 40, 55, 20))
        self.txt4_p2 = QLineEdit(self.Panel2)
        self.txt4_p2.setObjectName(u"txt4_p2")
        self.txt4_p2.setEnabled(False)
        self.txt4_p2.setGeometry(QRect(220, 40, 55, 20))
        self.txt9_p2 = QLineEdit(self.Panel2)
        self.txt9_p2.setObjectName(u"txt9_p2")
        self.txt9_p2.setEnabled(False)
        self.txt9_p2.setGeometry(QRect(220, 60, 55, 20))
        self.label26 = QLabel(self.Panel2)
        self.label26.setObjectName(u"label26")
        self.label26.setGeometry(QRect(10, 60, 31, 17))
        self.txt8_p2 = QLineEdit(self.Panel2)
        self.txt8_p2.setObjectName(u"txt8_p2")
        self.txt8_p2.setGeometry(QRect(160, 60, 55, 20))
        self.txt6_p2 = QLineEdit(self.Panel2)
        self.txt6_p2.setObjectName(u"txt6_p2")
        self.txt6_p2.setGeometry(QRect(40, 60, 55, 20))
        self.txt7_p2 = QLineEdit(self.Panel2)
        self.txt7_p2.setObjectName(u"txt7_p2")
        self.txt7_p2.setGeometry(QRect(100, 60, 55, 20))
        self.label27 = QLabel(self.Panel2)
        self.label27.setObjectName(u"label27")
        self.label27.setGeometry(QRect(10, 80, 31, 17))
        self.label28 = QLabel(self.Panel2)
        self.label28.setObjectName(u"label28")
        self.label28.setGeometry(QRect(10, 100, 31, 17))
        self.txt5_p2 = QLineEdit(self.Panel2)
        self.txt5_p2.setObjectName(u"txt5_p2")
        self.txt5_p2.setGeometry(QRect(290, 40, 91, 20))
        self.txt10_p2 = QLineEdit(self.Panel2)
        self.txt10_p2.setObjectName(u"txt10_p2")
        self.txt10_p2.setGeometry(QRect(290, 60, 91, 20))
        self.label19 = QLabel(self.Panel2)
        self.label19.setObjectName(u"label19")
        self.label19.setGeometry(QRect(40, 20, 55, 20))
        self.label19.setAlignment(Qt.AlignCenter)
        self.label20 = QLabel(self.Panel2)
        self.label20.setObjectName(u"label20")
        self.label20.setGeometry(QRect(100, 20, 55, 20))
        self.label20.setAlignment(Qt.AlignCenter)
        self.label21 = QLabel(self.Panel2)
        self.label21.setObjectName(u"label21")
        self.label21.setGeometry(QRect(160, 20, 55, 20))
        self.label21.setAlignment(Qt.AlignCenter)
        self.label23 = QLabel(self.Panel2)
        self.label23.setObjectName(u"label23")
        self.label23.setGeometry(QRect(220, 20, 55, 20))
        self.label23.setAlignment(Qt.AlignCenter)
        self.label24 = QLabel(self.Panel2)
        self.label24.setObjectName(u"label24")
        self.label24.setGeometry(QRect(290, 20, 91, 20))
        self.label24.setAlignment(Qt.AlignCenter)
        self.txt16_p2 = QLineEdit(self.Panel2)
        self.txt16_p2.setObjectName(u"txt16_p2")
        self.txt16_p2.setEnabled(False)
        self.txt16_p2.setGeometry(QRect(40, 100, 55, 20))
        self.txt18_p2 = QLineEdit(self.Panel2)
        self.txt18_p2.setObjectName(u"txt18_p2")
        self.txt18_p2.setEnabled(False)
        self.txt18_p2.setGeometry(QRect(160, 100, 55, 20))
        self.txt17_p2 = QLineEdit(self.Panel2)
        self.txt17_p2.setObjectName(u"txt17_p2")
        self.txt17_p2.setEnabled(False)
        self.txt17_p2.setGeometry(QRect(100, 100, 55, 20))
        self.txt14_p2 = QLineEdit(self.Panel2)
        self.txt14_p2.setObjectName(u"txt14_p2")
        self.txt14_p2.setEnabled(False)
        self.txt14_p2.setGeometry(QRect(220, 80, 55, 20))
        self.txt15_p2 = QLineEdit(self.Panel2)
        self.txt15_p2.setObjectName(u"txt15_p2")
        self.txt15_p2.setGeometry(QRect(290, 80, 91, 20))
        self.txt13_p2 = QLineEdit(self.Panel2)
        self.txt13_p2.setObjectName(u"txt13_p2")
        self.txt13_p2.setGeometry(QRect(160, 80, 55, 20))
        self.txt11_p2 = QLineEdit(self.Panel2)
        self.txt11_p2.setObjectName(u"txt11_p2")
        self.txt11_p2.setGeometry(QRect(40, 80, 55, 20))
        self.txt19_p2 = QLineEdit(self.Panel2)
        self.txt19_p2.setObjectName(u"txt19_p2")
        self.txt19_p2.setGeometry(QRect(220, 100, 55, 20))
        self.txt12_p2 = QLineEdit(self.Panel2)
        self.txt12_p2.setObjectName(u"txt12_p2")
        self.txt12_p2.setGeometry(QRect(100, 80, 55, 20))
        self.txt20_p2 = QLineEdit(self.Panel2)
        self.txt20_p2.setObjectName(u"txt20_p2")
        self.txt20_p2.setGeometry(QRect(290, 100, 91, 20))
        self.btProjectionAll = QPushButton(self.Panel2)
        self.btProjectionAll.setObjectName(u"btProjectionAll")
        self.btProjectionAll.setGeometry(QRect(458, 30, 91, 41))
        self.btProjectionClear = QPushButton(self.Panel2)
        self.btProjectionClear.setObjectName(u"btProjectionClear")
        self.btProjectionClear.setGeometry(QRect(480, 90, 60, 27))
        self.btProjectionClear.setFont(font)
        self.txt21_p2 = QLineEdit(self.Panel2)
        self.txt21_p2.setObjectName(u"txt21_p2")
        self.txt21_p2.setGeometry(QRect(390, 40, 61, 20))
        self.txt22_p2 = QLineEdit(self.Panel2)
        self.txt22_p2.setObjectName(u"txt22_p2")
        self.txt22_p2.setGeometry(QRect(390, 60, 61, 20))
        self.txt23_p2 = QLineEdit(self.Panel2)
        self.txt23_p2.setObjectName(u"txt23_p2")
        self.txt23_p2.setGeometry(QRect(390, 80, 61, 20))
        self.txt24_p2 = QLineEdit(self.Panel2)
        self.txt24_p2.setObjectName(u"txt24_p2")
        self.txt24_p2.setGeometry(QRect(390, 100, 61, 20))
        self.label24_2 = QLabel(self.Panel2)
        self.label24_2.setObjectName(u"label24_2")
        self.label24_2.setGeometry(QRect(390, 20, 61, 20))
        self.label24_2.setAlignment(Qt.AlignCenter)
        self.Panel3 = QGroupBox(self.centralwidget)
        self.Panel3.setObjectName(u"Panel3")
        self.Panel3.setGeometry(QRect(10, 540, 560, 151))
        self.Panel3.setFlat(False)
        self.label33 = QLabel(self.Panel3)
        self.label33.setObjectName(u"label33")
        self.label33.setGeometry(QRect(0, 60, 31, 17))
        self.txt1_p3 = QLineEdit(self.Panel3)
        self.txt1_p3.setObjectName(u"txt1_p3")
        self.txt1_p3.setGeometry(QRect(30, 60, 60, 20))
        self.label34 = QLabel(self.Panel3)
        self.label34.setObjectName(u"label34")
        self.label34.setGeometry(QRect(100, 60, 16, 17))
        self.txt2_p3 = QLineEdit(self.Panel3)
        self.txt2_p3.setObjectName(u"txt2_p3")
        self.txt2_p3.setGeometry(QRect(120, 60, 60, 20))
        self.btAutoFillAx1 = QPushButton(self.Panel3)
        self.btAutoFillAx1.setObjectName(u"btAutoFillAx1")
        self.btAutoFillAx1.setGeometry(QRect(190, 60, 32, 22))
        self.btAutoFillAx1.setFont(font)
        self.txtFold1 = QLineEdit(self.Panel3)
        self.txtFold1.setObjectName(u"txtFold1")
        self.txtFold1.setGeometry(QRect(240, 60, 40, 20))
        self.v1ch = QComboBox(self.Panel3)
        self.v1ch.addItem("")
        self.v1ch.addItem("")
        self.v1ch.addItem("")
        self.v1ch.addItem("")
        self.v1ch.setObjectName(u"v1ch")
        self.v1ch.setGeometry(QRect(380, 60, 100, 22))
        self.v1ch.setFont(font)
        self.txt3_p3 = QLineEdit(self.Panel3)
        self.txt3_p3.setObjectName(u"txt3_p3")
        self.txt3_p3.setGeometry(QRect(490, 60, 60, 20))
        self.btAutoFillAx2 = QPushButton(self.Panel3)
        self.btAutoFillAx2.setObjectName(u"btAutoFillAx2")
        self.btAutoFillAx2.setGeometry(QRect(190, 80, 32, 22))
        self.btAutoFillAx2.setFont(font)
        self.label36 = QLabel(self.Panel3)
        self.label36.setObjectName(u"label36")
        self.label36.setGeometry(QRect(100, 80, 16, 17))
        self.txt5_p3 = QLineEdit(self.Panel3)
        self.txt5_p3.setObjectName(u"txt5_p3")
        self.txt5_p3.setGeometry(QRect(120, 80, 60, 20))
        self.txt6_p3 = QLineEdit(self.Panel3)
        self.txt6_p3.setObjectName(u"txt6_p3")
        self.txt6_p3.setGeometry(QRect(490, 80, 60, 20))
        self.v2ch = QComboBox(self.Panel3)
        self.v2ch.addItem("")
        self.v2ch.addItem("")
        self.v2ch.addItem("")
        self.v2ch.addItem("")
        self.v2ch.setObjectName(u"v2ch")
        self.v2ch.setGeometry(QRect(380, 80, 100, 22))
        self.v2ch.setFont(font)
        self.label35 = QLabel(self.Panel3)
        self.label35.setObjectName(u"label35")
        self.label35.setGeometry(QRect(0, 80, 31, 17))
        self.txt4_p3 = QLineEdit(self.Panel3)
        self.txt4_p3.setObjectName(u"txt4_p3")
        self.txt4_p3.setGeometry(QRect(30, 80, 60, 20))
        self.txtFold2 = QLineEdit(self.Panel3)
        self.txtFold2.setObjectName(u"txtFold2")
        self.txtFold2.setGeometry(QRect(240, 80, 40, 20))
        self.btAutoFillAx3 = QPushButton(self.Panel3)
        self.btAutoFillAx3.setObjectName(u"btAutoFillAx3")
        self.btAutoFillAx3.setGeometry(QRect(190, 100, 32, 22))
        self.btAutoFillAx3.setFont(font)
        self.label38 = QLabel(self.Panel3)
        self.label38.setObjectName(u"label38")
        self.label38.setGeometry(QRect(100, 100, 16, 17))
        self.txt8_p3 = QLineEdit(self.Panel3)
        self.txt8_p3.setObjectName(u"txt8_p3")
        self.txt8_p3.setGeometry(QRect(120, 100, 60, 20))
        self.txt9_p3 = QLineEdit(self.Panel3)
        self.txt9_p3.setObjectName(u"txt9_p3")
        self.txt9_p3.setGeometry(QRect(490, 100, 60, 20))
        self.v3ch = QComboBox(self.Panel3)
        self.v3ch.addItem("")
        self.v3ch.addItem("")
        self.v3ch.addItem("")
        self.v3ch.addItem("")
        self.v3ch.setObjectName(u"v3ch")
        self.v3ch.setGeometry(QRect(380, 100, 100, 22))
        self.v3ch.setFont(font)
        self.label37 = QLabel(self.Panel3)
        self.label37.setObjectName(u"label37")
        self.label37.setGeometry(QRect(0, 100, 31, 17))
        self.txt7_p3 = QLineEdit(self.Panel3)
        self.txt7_p3.setObjectName(u"txt7_p3")
        self.txt7_p3.setGeometry(QRect(30, 100, 60, 20))
        self.txtFold3 = QLineEdit(self.Panel3)
        self.txtFold3.setObjectName(u"txtFold3")
        self.txtFold3.setGeometry(QRect(240, 100, 40, 20))
        self.btAutoFillAx4 = QPushButton(self.Panel3)
        self.btAutoFillAx4.setObjectName(u"btAutoFillAx4")
        self.btAutoFillAx4.setGeometry(QRect(190, 120, 32, 22))
        self.btAutoFillAx4.setFont(font)
        self.label40 = QLabel(self.Panel3)
        self.label40.setObjectName(u"label40")
        self.label40.setGeometry(QRect(100, 120, 16, 17))
        self.txt11_p3 = QLineEdit(self.Panel3)
        self.txt11_p3.setObjectName(u"txt11_p3")
        self.txt11_p3.setGeometry(QRect(120, 120, 60, 20))
        self.txt12_p3 = QLineEdit(self.Panel3)
        self.txt12_p3.setObjectName(u"txt12_p3")
        self.txt12_p3.setGeometry(QRect(490, 120, 60, 20))
        self.v4ch = QComboBox(self.Panel3)
        self.v4ch.addItem("")
        self.v4ch.addItem("")
        self.v4ch.addItem("")
        self.v4ch.addItem("")
        self.v4ch.setObjectName(u"v4ch")
        self.v4ch.setGeometry(QRect(380, 120, 100, 22))
        self.v4ch.setFont(font)
        self.label39 = QLabel(self.Panel3)
        self.label39.setObjectName(u"label39")
        self.label39.setGeometry(QRect(0, 120, 31, 17))
        self.txt10_p3 = QLineEdit(self.Panel3)
        self.txt10_p3.setObjectName(u"txt10_p3")
        self.txt10_p3.setGeometry(QRect(30, 120, 60, 20))
        self.txtFold4 = QLineEdit(self.Panel3)
        self.txtFold4.setObjectName(u"txtFold4")
        self.txtFold4.setGeometry(QRect(240, 120, 40, 20))
        self.btAutoFill = QPushButton(self.Panel3)
        self.btAutoFill.setObjectName(u"btAutoFill")
        self.btAutoFill.setGeometry(QRect(130, 25, 51, 27))
        self.btAutoFill.setFont(font)
        self.label29 = QLabel(self.Panel3)
        self.label29.setObjectName(u"label29")
        self.label29.setGeometry(QRect(80, 30, 51, 17))
        self.label30 = QLabel(self.Panel3)
        self.label30.setObjectName(u"label30")
        self.label30.setGeometry(QRect(240, 30, 61, 17))
        self.cbDiagFoldType = QComboBox(self.Panel3)
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.setObjectName(u"cbDiagFoldType")
        self.cbDiagFoldType.setGeometry(QRect(290, 30, 71, 21))
        self.cbDiagFoldType.setFont(font)
        self.label31 = QLabel(self.Panel3)
        self.label31.setObjectName(u"label31")
        self.label31.setGeometry(QRect(390, 30, 81, 17))
        self.label32 = QLabel(self.Panel3)
        self.label32.setObjectName(u"label32")
        self.label32.setGeometry(QRect(500, 30, 51, 17))
        self.cbDiagFoldAx1 = QComboBox(self.Panel3)
        self.cbDiagFoldAx1.addItem("")
        self.cbDiagFoldAx1.addItem("")
        self.cbDiagFoldAx1.addItem("")
        self.cbDiagFoldAx1.setObjectName(u"cbDiagFoldAx1")
        self.cbDiagFoldAx1.setGeometry(QRect(300, 60, 51, 22))
        self.cbDiagFoldAx1.setFont(font)
        self.cbDiagFoldAx2 = QComboBox(self.Panel3)
        self.cbDiagFoldAx2.addItem("")
        self.cbDiagFoldAx2.addItem("")
        self.cbDiagFoldAx2.addItem("")
        self.cbDiagFoldAx2.setObjectName(u"cbDiagFoldAx2")
        self.cbDiagFoldAx2.setGeometry(QRect(300, 80, 51, 22))
        self.cbDiagFoldAx2.setFont(font)
        self.cbDiagFoldAx3 = QComboBox(self.Panel3)
        self.cbDiagFoldAx3.addItem("")
        self.cbDiagFoldAx3.addItem("")
        self.cbDiagFoldAx3.addItem("")
        self.cbDiagFoldAx3.setObjectName(u"cbDiagFoldAx3")
        self.cbDiagFoldAx3.setGeometry(QRect(300, 100, 51, 22))
        self.cbDiagFoldAx3.setFont(font)
        self.cbDiagFoldAx4 = QComboBox(self.Panel3)
        self.cbDiagFoldAx4.addItem("")
        self.cbDiagFoldAx4.addItem("")
        self.cbDiagFoldAx4.addItem("")
        self.cbDiagFoldAx4.setObjectName(u"cbDiagFoldAx4")
        self.cbDiagFoldAx4.setGeometry(QRect(300, 120, 51, 22))
        self.cbDiagFoldAx4.setFont(font)
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(10, 690, 231, 51))
        self.btDataConversion = QPushButton(self.groupBox)
        self.btDataConversion.setObjectName(u"btDataConversion")
        self.btDataConversion.setGeometry(QRect(140, 24, 81, 25))
        self.cbDataConversions = QComboBox(self.groupBox)
        self.cbDataConversions.addItem("")
        self.cbDataConversions.addItem("")
        self.cbDataConversions.addItem("")
        self.cbDataConversions.addItem("")
        self.cbDataConversions.setObjectName(u"cbDataConversions")
        self.cbDataConversions.setGeometry(QRect(10, 24, 111, 25))
        self.btBack = QToolButton(self.centralwidget)
        self.btBack.setObjectName(u"btBack")
        self.btBack.setGeometry(QRect(260, 710, 24, 25))
        self.btBack.setArrowType(Qt.LeftArrow)
        self.btForward = QToolButton(self.centralwidget)
        self.btForward.setObjectName(u"btForward")
        self.btForward.setGeometry(QRect(290, 710, 24, 25))
        self.btForward.setArrowType(Qt.RightArrow)
        self.btSlice = QPushButton(self.centralwidget)
        self.btSlice.setObjectName(u"btSlice")
        self.btSlice.setGeometry(QRect(340, 710, 99, 27))
        self.btSlice3D = QPushButton(self.centralwidget)
        self.btSlice3D.setObjectName(u"btSlice3D")
        self.btSlice3D.setGeometry(QRect(450, 710, 99, 27))
        self.gbDataReduct = QGroupBox(self.centralwidget)
        self.gbDataReduct.setObjectName(u"gbDataReduct")
        self.gbDataReduct.setGeometry(QRect(10, 60, 151, 61))
        self.btDRExec = QPushButton(self.gbDataReduct)
        self.btDRExec.setObjectName(u"btDRExec")
        self.btDRExec.setGeometry(QRect(10, 26, 131, 31))
        self.gbExperiment = QGroupBox(self.centralwidget)
        self.gbExperiment.setObjectName(u"gbExperiment")
        self.gbExperiment.setGeometry(QRect(10, 0, 151, 61))
        self.btDataReduction = QPushButton(self.gbExperiment)
        self.btDataReduction.setObjectName(u"btDataReduction")
        self.btDataReduction.setGeometry(QRect(10, 25, 131, 31))
        self.gbOnLineBox = QGroupBox(self.centralwidget)
        self.gbOnLineBox.setObjectName(u"gbOnLineBox")
        self.gbOnLineBox.setGeometry(QRect(10, 130, 560, 91))
        self.ckAUwithReduction = QCheckBox(self.gbOnLineBox)
        self.ckAUwithReduction.setObjectName(u"ckAUwithReduction")
        self.ckAUwithReduction.setGeometry(QRect(170, 25, 141, 22))
        self.ckAUwithProjection = QCheckBox(self.gbOnLineBox)
        self.ckAUwithProjection.setObjectName(u"ckAUwithProjection")
        self.ckAUwithProjection.setGeometry(QRect(170, 45, 141, 22))
        self.ckAUwithSlice = QCheckBox(self.gbOnLineBox)
        self.ckAUwithSlice.setObjectName(u"ckAUwithSlice")
        self.ckAUwithSlice.setGeometry(QRect(170, 65, 141, 22))
        self.btAUupdate = QPushButton(self.gbOnLineBox)
        self.btAUupdate.setObjectName(u"btAUupdate")
        self.btAUupdate.setGeometry(QRect(320, 25, 99, 27))
        self.btAUstart = QPushButton(self.gbOnLineBox)
        self.btAUstart.setObjectName(u"btAUstart")
        self.btAUstart.setGeometry(QRect(320, 55, 111, 27))
        self.txAUIntervalTime = QLineEdit(self.gbOnLineBox)
        self.txAUIntervalTime.setObjectName(u"txAUIntervalTime")
        self.txAUIntervalTime.setGeometry(QRect(440, 55, 71, 27))
        self.txtAUunit = QLabel(self.gbOnLineBox)
        self.txtAUunit.setObjectName(u"txtAUunit")
        self.txtAUunit.setGeometry(QRect(520, 65, 41, 17))
        self.tgbOnLineMode = QToolButton(self.gbOnLineBox)
        self.tgbOnLineMode.setObjectName(u"tgbOnLineMode")
        self.tgbOnLineMode.setGeometry(QRect(10, 30, 121, 51))
        self.tgbOnLineMode.setCheckable(True)
        self.panel5 = QGroupBox(self.centralwidget)
        self.panel5.setObjectName(u"panel5")
        self.panel5.setGeometry(QRect(200, 0, 191, 61))
        self.btLoadData = QPushButton(self.panel5)
        self.btLoadData.setObjectName(u"btLoadData")
        self.btLoadData.setGeometry(QRect(9, 30, 61, 25))
        self.btLoadData.setFont(font)
        self.btSaveData = QPushButton(self.panel5)
        self.btSaveData.setObjectName(u"btSaveData")
        self.btSaveData.setGeometry(QRect(80, 30, 41, 25))
        self.btSaveData.setFont(font)
        self.cbSaveDataWithComp = QCheckBox(self.panel5)
        self.cbSaveDataWithComp.setObjectName(u"cbSaveDataWithComp")
        self.cbSaveDataWithComp.setGeometry(QRect(130, 30, 92, 23))
        self.cbSaveDataWithComp.setFont(font)
        self.txtExpInfo = QLabel(self.centralwidget)
        self.txtExpInfo.setObjectName(u"txtExpInfo")
        self.txtExpInfo.setGeometry(QRect(190, 90, 381, 41))
        self.txtRunNo = QLabel(self.centralwidget)
        self.txtRunNo.setObjectName(u"txtRunNo")
        self.txtRunNo.setGeometry(QRect(250, 70, 311, 20))
        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(180, 70, 68, 20))
        self.panel5_2 = QGroupBox(self.centralwidget)
        self.panel5_2.setObjectName(u"panel5_2")
        self.panel5_2.setGeometry(QRect(400, 10, 171, 51))
        self.btLoadXtalParam = QPushButton(self.panel5_2)
        self.btLoadXtalParam.setObjectName(u"btLoadXtalParam")
        self.btLoadXtalParam.setGeometry(QRect(10, 24, 41, 25))
        self.btLoadXtalParam.setFont(font)
        self.btSaveXtalParam = QPushButton(self.panel5_2)
        self.btSaveXtalParam.setObjectName(u"btSaveXtalParam")
        self.btSaveXtalParam.setGeometry(QRect(60, 24, 41, 25))
        self.btSaveXtalParam.setFont(font)
        self.btAdvXtalParam = QPushButton(self.panel5_2)
        self.btAdvXtalParam.setObjectName(u"btAdvXtalParam")
        self.btAdvXtalParam.setGeometry(QRect(110, 24, 50, 25))
        self.btAdvXtalParam.setFont(font)
        self.ckMemorySaveMode = QCheckBox(self.centralwidget)
        self.ckMemorySaveMode.setObjectName(u"ckMemorySaveMode")
        self.ckMemorySaveMode.setGeometry(QRect(410, 390, 171, 22))
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 582, 22))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.btDataReduction, self.btLoadData)
        QWidget.setTabOrder(self.btLoadData, self.btSaveData)
        QWidget.setTabOrder(self.btSaveData, self.cbSaveDataWithComp)
        QWidget.setTabOrder(self.cbSaveDataWithComp, self.btLoadXtalParam)
        QWidget.setTabOrder(self.btLoadXtalParam, self.btSaveXtalParam)
        QWidget.setTabOrder(self.btSaveXtalParam, self.btAdvXtalParam)
        QWidget.setTabOrder(self.btAdvXtalParam, self.btDRExec)
        QWidget.setTabOrder(self.btDRExec, self.txt1_p1)
        QWidget.setTabOrder(self.txt1_p1, self.txt2_p1)
        QWidget.setTabOrder(self.txt2_p1, self.txt3_p1)
        QWidget.setTabOrder(self.txt3_p1, self.txt4_p1)
        QWidget.setTabOrder(self.txt4_p1, self.txt5_p1)
        QWidget.setTabOrder(self.txt5_p1, self.txt6_p1)
        QWidget.setTabOrder(self.txt6_p1, self.txt7_p1)
        QWidget.setTabOrder(self.txt7_p1, self.txt8_p1)
        QWidget.setTabOrder(self.txt8_p1, self.txt9_p1)
        QWidget.setTabOrder(self.txt9_p1, self.txt10_p1)
        QWidget.setTabOrder(self.txt10_p1, self.txt11_p1)
        QWidget.setTabOrder(self.txt11_p1, self.txt12_p1)
        QWidget.setTabOrder(self.txt12_p1, self.chRotateAxis1st)
        QWidget.setTabOrder(self.chRotateAxis1st, self.txt13_p1)
        QWidget.setTabOrder(self.txt13_p1, self.chRotateAxis2nd)
        QWidget.setTabOrder(self.chRotateAxis2nd, self.txt14_p1)
        QWidget.setTabOrder(self.txt14_p1, self.chRotateAxis3rd)
        QWidget.setTabOrder(self.chRotateAxis3rd, self.txt15_p1)
        QWidget.setTabOrder(self.txt15_p1, self.ckMemorySaveMode)
        QWidget.setTabOrder(self.ckMemorySaveMode, self.txt1_p2)
        QWidget.setTabOrder(self.txt1_p2, self.txt2_p2)
        QWidget.setTabOrder(self.txt2_p2, self.txt3_p2)
        QWidget.setTabOrder(self.txt3_p2, self.txt4_p2)
        QWidget.setTabOrder(self.txt4_p2, self.txt5_p2)
        QWidget.setTabOrder(self.txt5_p2, self.txt21_p2)
        QWidget.setTabOrder(self.txt21_p2, self.txt6_p2)
        QWidget.setTabOrder(self.txt6_p2, self.txt7_p2)
        QWidget.setTabOrder(self.txt7_p2, self.txt8_p2)
        QWidget.setTabOrder(self.txt8_p2, self.txt9_p2)
        QWidget.setTabOrder(self.txt9_p2, self.txt10_p2)
        QWidget.setTabOrder(self.txt10_p2, self.txt22_p2)
        QWidget.setTabOrder(self.txt22_p2, self.txt11_p2)
        QWidget.setTabOrder(self.txt11_p2, self.txt12_p2)
        QWidget.setTabOrder(self.txt12_p2, self.txt13_p2)
        QWidget.setTabOrder(self.txt13_p2, self.txt14_p2)
        QWidget.setTabOrder(self.txt14_p2, self.txt15_p2)
        QWidget.setTabOrder(self.txt15_p2, self.txt23_p2)
        QWidget.setTabOrder(self.txt23_p2, self.txt16_p2)
        QWidget.setTabOrder(self.txt16_p2, self.txt17_p2)
        QWidget.setTabOrder(self.txt17_p2, self.txt18_p2)
        QWidget.setTabOrder(self.txt18_p2, self.txt19_p2)
        QWidget.setTabOrder(self.txt19_p2, self.txt20_p2)
        QWidget.setTabOrder(self.txt20_p2, self.txt24_p2)
        QWidget.setTabOrder(self.txt24_p2, self.btProjectionAll)
        QWidget.setTabOrder(self.btProjectionAll, self.btProjectionClear)
        QWidget.setTabOrder(self.btProjectionClear, self.btAutoFill)
        QWidget.setTabOrder(self.btAutoFill, self.cbDiagFoldType)
        QWidget.setTabOrder(self.cbDiagFoldType, self.txt1_p3)
        QWidget.setTabOrder(self.txt1_p3, self.txt2_p3)
        QWidget.setTabOrder(self.txt2_p3, self.btAutoFillAx1)
        QWidget.setTabOrder(self.btAutoFillAx1, self.txtFold1)
        QWidget.setTabOrder(self.txtFold1, self.cbDiagFoldAx1)
        QWidget.setTabOrder(self.cbDiagFoldAx1, self.v1ch)
        QWidget.setTabOrder(self.v1ch, self.txt3_p3)
        QWidget.setTabOrder(self.txt3_p3, self.txt4_p3)
        QWidget.setTabOrder(self.txt4_p3, self.txt5_p3)
        QWidget.setTabOrder(self.txt5_p3, self.btAutoFillAx2)
        QWidget.setTabOrder(self.btAutoFillAx2, self.txtFold2)
        QWidget.setTabOrder(self.txtFold2, self.cbDiagFoldAx2)
        QWidget.setTabOrder(self.cbDiagFoldAx2, self.v2ch)
        QWidget.setTabOrder(self.v2ch, self.txt6_p3)
        QWidget.setTabOrder(self.txt6_p3, self.txt7_p3)
        QWidget.setTabOrder(self.txt7_p3, self.txt8_p3)
        QWidget.setTabOrder(self.txt8_p3, self.btAutoFillAx3)
        QWidget.setTabOrder(self.btAutoFillAx3, self.txtFold3)
        QWidget.setTabOrder(self.txtFold3, self.cbDiagFoldAx3)
        QWidget.setTabOrder(self.cbDiagFoldAx3, self.v3ch)
        QWidget.setTabOrder(self.v3ch, self.txt9_p3)
        QWidget.setTabOrder(self.txt9_p3, self.txt10_p3)
        QWidget.setTabOrder(self.txt10_p3, self.txt11_p3)
        QWidget.setTabOrder(self.txt11_p3, self.btAutoFillAx4)
        QWidget.setTabOrder(self.btAutoFillAx4, self.txtFold4)
        QWidget.setTabOrder(self.txtFold4, self.cbDiagFoldAx4)
        QWidget.setTabOrder(self.cbDiagFoldAx4, self.v4ch)
        QWidget.setTabOrder(self.v4ch, self.txt12_p3)
        QWidget.setTabOrder(self.txt12_p3, self.btSlice)
        QWidget.setTabOrder(self.btSlice, self.btSlice3D)
        QWidget.setTabOrder(self.btSlice3D, self.btBack)
        QWidget.setTabOrder(self.btBack, self.btForward)
        QWidget.setTabOrder(self.btForward, self.cbDataConversions)
        QWidget.setTabOrder(self.cbDataConversions, self.btDataConversion)
        QWidget.setTabOrder(self.btDataConversion, self.tgbOnLineMode)
        QWidget.setTabOrder(self.tgbOnLineMode, self.ckAUwithReduction)
        QWidget.setTabOrder(self.ckAUwithReduction, self.ckAUwithProjection)
        QWidget.setTabOrder(self.ckAUwithProjection, self.ckAUwithSlice)
        QWidget.setTabOrder(self.ckAUwithSlice, self.btAUupdate)
        QWidget.setTabOrder(self.btAUupdate, self.btAUstart)
        QWidget.setTabOrder(self.btAUstart, self.txAUIntervalTime)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"D4Mat2Slicer Main", None))
        self.Panel1.setTitle(QCoreApplication.translate("MainWindow", u"Sample Information", None))
        self.txt14_p1.setText("")
        self.label15.setText(QCoreApplication.translate("MainWindow", u"Rotation", None))
        self.label7.setText(QCoreApplication.translate("MainWindow", u"beta:", None))
        self.label4.setText(QCoreApplication.translate("MainWindow", u"b (Ang):", None))
        self.label6.setText(QCoreApplication.translate("MainWindow", u"alpha:", None))
        self.label14.setText(QCoreApplication.translate("MainWindow", u"Vl:", None))
        self.label18.setText(QCoreApplication.translate("MainWindow", u"3rd:", None))
        self.label8.setText(QCoreApplication.translate("MainWindow", u"gamma:", None))
        self.chRotateAxis3rd.setItemText(0, "")
        self.chRotateAxis3rd.setItemText(1, QCoreApplication.translate("MainWindow", u"Y", None))
        self.chRotateAxis3rd.setItemText(2, QCoreApplication.translate("MainWindow", u"Z", None))
        self.chRotateAxis3rd.setItemText(3, QCoreApplication.translate("MainWindow", u"X", None))

        self.label17.setText(QCoreApplication.translate("MainWindow", u"2nd:", None))
        self.label11.setText(QCoreApplication.translate("MainWindow", u"Ul:", None))
        self.label10.setText(QCoreApplication.translate("MainWindow", u"Uk:", None))
        self.chRotateAxis2nd.setItemText(0, "")
        self.chRotateAxis2nd.setItemText(1, QCoreApplication.translate("MainWindow", u"Y", None))
        self.chRotateAxis2nd.setItemText(2, QCoreApplication.translate("MainWindow", u"Z", None))
        self.chRotateAxis2nd.setItemText(3, QCoreApplication.translate("MainWindow", u"X", None))

        self.label13.setText(QCoreApplication.translate("MainWindow", u"Vk:", None))
        self.chRotateAxis1st.setItemText(0, "")
        self.chRotateAxis1st.setItemText(1, QCoreApplication.translate("MainWindow", u"Y", None))
        self.chRotateAxis1st.setItemText(2, QCoreApplication.translate("MainWindow", u"Z", None))
        self.chRotateAxis1st.setItemText(3, QCoreApplication.translate("MainWindow", u"X", None))

        self.label3.setText(QCoreApplication.translate("MainWindow", u"a (Ang):", None))
        self.label16.setText(QCoreApplication.translate("MainWindow", u"1st:", None))
        self.label5.setText(QCoreApplication.translate("MainWindow", u"c (Ang):", None))
        self.label9.setText(QCoreApplication.translate("MainWindow", u"Uh:", None))
        self.label12.setText(QCoreApplication.translate("MainWindow", u"Vh:", None))
        self.Panel2.setTitle(QCoreApplication.translate("MainWindow", u"Projection", None))
        self.label25.setText(QCoreApplication.translate("MainWindow", u"ax1:", None))
        self.label26.setText(QCoreApplication.translate("MainWindow", u"ax2:", None))
        self.label27.setText(QCoreApplication.translate("MainWindow", u"ax3:", None))
        self.label28.setText(QCoreApplication.translate("MainWindow", u"ax4:", None))
        self.label19.setText(QCoreApplication.translate("MainWindow", u"a*", None))
        self.label20.setText(QCoreApplication.translate("MainWindow", u"b*", None))
        self.label21.setText(QCoreApplication.translate("MainWindow", u"c*", None))
        self.label23.setText(QCoreApplication.translate("MainWindow", u"Energy", None))
        self.label24.setText(QCoreApplication.translate("MainWindow", u"Axis Label", None))
        self.btProjectionAll.setText(QCoreApplication.translate("MainWindow", u"Projection", None))
        self.btProjectionClear.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.label24_2.setText(QCoreApplication.translate("MainWindow", u"Unit", None))
        self.Panel3.setTitle(QCoreApplication.translate("MainWindow", u"Slicing", None))
        self.label33.setText(QCoreApplication.translate("MainWindow", u"ax1:", None))
        self.label34.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.btAutoFillAx1.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.v1ch.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.v1ch.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.v1ch.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))
        self.v1ch.setItemText(3, QCoreApplication.translate("MainWindow", u"Z-Axis", None))

        self.btAutoFillAx2.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label36.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.v2ch.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.v2ch.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.v2ch.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))
        self.v2ch.setItemText(3, QCoreApplication.translate("MainWindow", u"Z-Axis", None))

        self.label35.setText(QCoreApplication.translate("MainWindow", u"ax2:", None))
        self.btAutoFillAx3.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label38.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.v3ch.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.v3ch.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.v3ch.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))
        self.v3ch.setItemText(3, QCoreApplication.translate("MainWindow", u"Z-Axis", None))

        self.label37.setText(QCoreApplication.translate("MainWindow", u"ax3:", None))
        self.btAutoFillAx4.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label40.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.v4ch.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.v4ch.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.v4ch.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))
        self.v4ch.setItemText(3, QCoreApplication.translate("MainWindow", u"Z-Axis", None))

        self.label39.setText(QCoreApplication.translate("MainWindow", u"ax4:", None))
        self.btAutoFill.setText(QCoreApplication.translate("MainWindow", u"Auto", None))
        self.label29.setText(QCoreApplication.translate("MainWindow", u"Range", None))
        self.label30.setText(QCoreApplication.translate("MainWindow", u"Folding", None))
        self.cbDiagFoldType.setItemText(0, QCoreApplication.translate("MainWindow", u"None", None))
        self.cbDiagFoldType.setItemText(1, QCoreApplication.translate("MainWindow", u"X' > Y'", None))
        self.cbDiagFoldType.setItemText(2, QCoreApplication.translate("MainWindow", u"X' < -Y'", None))
        self.cbDiagFoldType.setItemText(3, QCoreApplication.translate("MainWindow", u"Both", None))

        self.label31.setText(QCoreApplication.translate("MainWindow", u"Plot Axis", None))
        self.label32.setText(QCoreApplication.translate("MainWindow", u"Step", None))
        self.cbDiagFoldAx1.setItemText(0, QCoreApplication.translate("MainWindow", u"--", None))
        self.cbDiagFoldAx1.setItemText(1, QCoreApplication.translate("MainWindow", u"X'", None))
        self.cbDiagFoldAx1.setItemText(2, QCoreApplication.translate("MainWindow", u"Y'", None))

        self.cbDiagFoldAx2.setItemText(0, QCoreApplication.translate("MainWindow", u"--", None))
        self.cbDiagFoldAx2.setItemText(1, QCoreApplication.translate("MainWindow", u"X'", None))
        self.cbDiagFoldAx2.setItemText(2, QCoreApplication.translate("MainWindow", u"Y'", None))

        self.cbDiagFoldAx3.setItemText(0, QCoreApplication.translate("MainWindow", u"--", None))
        self.cbDiagFoldAx3.setItemText(1, QCoreApplication.translate("MainWindow", u"X'", None))
        self.cbDiagFoldAx3.setItemText(2, QCoreApplication.translate("MainWindow", u"Y'", None))

        self.cbDiagFoldAx4.setItemText(0, QCoreApplication.translate("MainWindow", u"--", None))
        self.cbDiagFoldAx4.setItemText(1, QCoreApplication.translate("MainWindow", u"X'", None))
        self.cbDiagFoldAx4.setItemText(2, QCoreApplication.translate("MainWindow", u"Y'", None))

        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"Data Conversion", None))
        self.btDataConversion.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.cbDataConversions.setItemText(0, QCoreApplication.translate("MainWindow", u"D4Mat", None))
        self.cbDataConversions.setItemText(1, QCoreApplication.translate("MainWindow", u"SPE", None))
        self.cbDataConversions.setItemText(2, QCoreApplication.translate("MainWindow", u"NxSPE", None))
        self.cbDataConversions.setItemText(3, QCoreApplication.translate("MainWindow", u"DumpAsText", None))

        self.btBack.setText(QCoreApplication.translate("MainWindow", u"...", None))
        self.btForward.setText(QCoreApplication.translate("MainWindow", u"...", None))
        self.btSlice.setText(QCoreApplication.translate("MainWindow", u"Slice", None))
        self.btSlice3D.setText(QCoreApplication.translate("MainWindow", u"3D Slice", None))
        self.gbDataReduct.setTitle(QCoreApplication.translate("MainWindow", u"Data Reduction", None))
        self.btDRExec.setText(QCoreApplication.translate("MainWindow", u"Execute", None))
        self.gbExperiment.setTitle(QCoreApplication.translate("MainWindow", u"Experiment", None))
        self.btDataReduction.setText(QCoreApplication.translate("MainWindow", u"Setting", None))
        self.gbOnLineBox.setTitle(QCoreApplication.translate("MainWindow", u"Online Data Reduction", None))
        self.ckAUwithReduction.setText(QCoreApplication.translate("MainWindow", u"with Reduction", None))
        self.ckAUwithProjection.setText(QCoreApplication.translate("MainWindow", u"with Projection", None))
        self.ckAUwithSlice.setText(QCoreApplication.translate("MainWindow", u"with Slice", None))
        self.btAUupdate.setText(QCoreApplication.translate("MainWindow", u"Update", None))
        self.btAUstart.setText(QCoreApplication.translate("MainWindow", u"Interval Start", None))
        self.txtAUunit.setText(QCoreApplication.translate("MainWindow", u"[sec]", None))
        self.tgbOnLineMode.setText(QCoreApplication.translate("MainWindow", u"On-Line Mode", None))
        self.panel5.setTitle(QCoreApplication.translate("MainWindow", u"D4Mat2 File ...", None))
        self.btLoadData.setText(QCoreApplication.translate("MainWindow", u"Load", None))
        self.btSaveData.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.cbSaveDataWithComp.setText(QCoreApplication.translate("MainWindow", u"Comp", None))
        self.txtExpInfo.setText("")
        self.txtRunNo.setText("")
        self.label.setText(QCoreApplication.translate("MainWindow", u"RunNo :", None))
        self.panel5_2.setTitle(QCoreApplication.translate("MainWindow", u"XtalParam File ...", None))
        self.btLoadXtalParam.setText(QCoreApplication.translate("MainWindow", u"Load", None))
        self.btSaveXtalParam.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.btAdvXtalParam.setText(QCoreApplication.translate("MainWindow", u"Partial", None))
        self.ckMemorySaveMode.setText(QCoreApplication.translate("MainWindow", u"Memory Save Mode", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'D4mat2SlicerQ_Sub.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGroupBox, QHeaderView,
    QLabel, QLineEdit, QMainWindow, QMenuBar,
    QPushButton, QRadioButton, QSizePolicy, QStatusBar,
    QTreeWidget, QTreeWidgetItem, QWidget)

class Ui_DataReductionPanel(object):
    def setupUi(self, DataReductionPanel):
        if not DataReductionPanel.objectName():
            DataReductionPanel.setObjectName(u"DataReductionPanel")
        DataReductionPanel.resize(522, 841)
        self.centralwidget = QWidget(DataReductionPanel)
        self.centralwidget.setObjectName(u"centralwidget")
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(10, 0, 361, 51))
        self.rbContRotMethod = QRadioButton(self.groupBox)
        self.rbContRotMethod.setObjectName(u"rbContRotMethod")
        self.rbContRotMethod.setGeometry(QRect(10, 25, 117, 22))
        self.rbStepByStepMethod = QRadioButton(self.groupBox)
        self.rbStepByStepMethod.setObjectName(u"rbStepByStepMethod")
        self.rbStepByStepMethod.setGeometry(QRect(220, 25, 117, 22))
        self.groupBox_2 = QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setGeometry(QRect(10, 50, 501, 61))
        self.txtEi = QLineEdit(self.groupBox_2)
        self.txtEi.setObjectName(u"txtEi")
        self.txtEi.setGeometry(QRect(40, 30, 71, 25))
        self.txtEnergyBin = QLineEdit(self.groupBox_2)
        self.txtEnergyBin.setObjectName(u"txtEnergyBin")
        self.txtEnergyBin.setGeometry(QRect(180, 30, 61, 25))
        self.txtEnergyMin = QLineEdit(self.groupBox_2)
        self.txtEnergyMin.setObjectName(u"txtEnergyMin")
        self.txtEnergyMin.setGeometry(QRect(330, 30, 61, 25))
        self.txtEnergyMax = QLineEdit(self.groupBox_2)
        self.txtEnergyMax.setObjectName(u"txtEnergyMax")
        self.txtEnergyMax.setGeometry(QRect(420, 30, 61, 25))
        self.label = QLabel(self.groupBox_2)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 30, 68, 25))
        self.label_2 = QLabel(self.groupBox_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(120, 30, 68, 25))
        self.label_3 = QLabel(self.groupBox_2)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(250, 30, 81, 25))
        self.label_4 = QLabel(self.groupBox_2)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(390, 30, 31, 25))
        self.label_4.setAlignment(Qt.AlignCenter)
        self.groupBox_3 = QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setGeometry(QRect(10, 110, 501, 51))
        self.txMaskFilePath = QLineEdit(self.groupBox_3)
        self.txMaskFilePath.setObjectName(u"txMaskFilePath")
        self.txMaskFilePath.setGeometry(QRect(20, 25, 381, 20))
        self.btSelectMaskFile = QPushButton(self.groupBox_3)
        self.btSelectMaskFile.setObjectName(u"btSelectMaskFile")
        self.btSelectMaskFile.setGeometry(QRect(420, 25, 61, 25))
        self.groupBox_4 = QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setGeometry(QRect(10, 250, 501, 71))
        self.rbUseDRDefault = QRadioButton(self.groupBox_4)
        self.rbUseDRDefault.setObjectName(u"rbUseDRDefault")
        self.rbUseDRDefault.setGeometry(QRect(10, 20, 117, 22))
        self.rbUseDRScript = QRadioButton(self.groupBox_4)
        self.rbUseDRScript.setObjectName(u"rbUseDRScript")
        self.rbUseDRScript.setGeometry(QRect(100, 20, 101, 22))
        self.btMakeDRTemp = QPushButton(self.groupBox_4)
        self.btMakeDRTemp.setObjectName(u"btMakeDRTemp")
        self.btMakeDRTemp.setGeometry(QRect(230, 20, 71, 27))
        self.btSelectDRScript = QPushButton(self.groupBox_4)
        self.btSelectDRScript.setObjectName(u"btSelectDRScript")
        self.btSelectDRScript.setGeometry(QRect(360, 20, 71, 27))
        self.label_5 = QLabel(self.groupBox_4)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(70, 50, 68, 17))
        self.lbDRScriptPath = QLabel(self.groupBox_4)
        self.lbDRScriptPath.setObjectName(u"lbDRScriptPath")
        self.lbDRScriptPath.setGeometry(QRect(130, 50, 351, 20))
        self.gbContRotMethod = QGroupBox(self.centralwidget)
        self.gbContRotMethod.setObjectName(u"gbContRotMethod")
        self.gbContRotMethod.setGeometry(QRect(20, 330, 231, 461))
        self.groupBox_7 = QGroupBox(self.gbContRotMethod)
        self.groupBox_7.setObjectName(u"groupBox_7")
        self.groupBox_7.setGeometry(QRect(10, 30, 211, 61))
        self.label_6 = QLabel(self.groupBox_7)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(10, 23, 68, 25))
        self.txtRunNo = QLineEdit(self.groupBox_7)
        self.txtRunNo.setObjectName(u"txtRunNo")
        self.txtRunNo.setGeometry(QRect(60, 25, 141, 25))
        self.groupBox_8 = QGroupBox(self.gbContRotMethod)
        self.groupBox_8.setObjectName(u"groupBox_8")
        self.groupBox_8.setGeometry(QRect(10, 100, 211, 151))
        self.label_7 = QLabel(self.groupBox_8)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(10, 28, 81, 25))
        self.label_8 = QLabel(self.groupBox_8)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(10, 90, 81, 25))
        self.label_9 = QLabel(self.groupBox_8)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(10, 120, 81, 25))
        self.txtStartDeg = QLineEdit(self.groupBox_8)
        self.txtStartDeg.setObjectName(u"txtStartDeg")
        self.txtStartDeg.setGeometry(QRect(90, 30, 111, 25))
        self.txtMaxDeg = QLineEdit(self.groupBox_8)
        self.txtMaxDeg.setObjectName(u"txtMaxDeg")
        self.txtMaxDeg.setGeometry(QRect(110, 90, 81, 25))
        self.txtStepDeg = QLineEdit(self.groupBox_8)
        self.txtStepDeg.setObjectName(u"txtStepDeg")
        self.txtStepDeg.setGeometry(QRect(110, 120, 81, 25))
        self.label_16 = QLabel(self.groupBox_8)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(10, 60, 81, 25))
        self.txtMinDeg = QLineEdit(self.groupBox_8)
        self.txtMinDeg.setObjectName(u"txtMinDeg")
        self.txtMinDeg.setGeometry(QRect(110, 60, 81, 25))
        self.groupBox_9 = QGroupBox(self.gbContRotMethod)
        self.groupBox_9.setObjectName(u"groupBox_9")
        self.groupBox_9.setGeometry(QRect(10, 260, 211, 51))
        self.label_10 = QLabel(self.groupBox_9)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(80, 20, 21, 25))
        self.label_11 = QLabel(self.groupBox_9)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(170, 20, 41, 25))
        self.txtTimeSliceStart = QLineEdit(self.groupBox_9)
        self.txtTimeSliceStart.setObjectName(u"txtTimeSliceStart")
        self.txtTimeSliceStart.setGeometry(QRect(0, 20, 71, 25))
        self.txtTimeSliceEnd = QLineEdit(self.groupBox_9)
        self.txtTimeSliceEnd.setObjectName(u"txtTimeSliceEnd")
        self.txtTimeSliceEnd.setGeometry(QRect(100, 20, 71, 25))
        self.gbTrigNETinfo = QGroupBox(self.gbContRotMethod)
        self.gbTrigNETinfo.setObjectName(u"gbTrigNETinfo")
        self.gbTrigNETinfo.setGeometry(QRect(10, 380, 211, 71))
        self.txtTrigNETdaqId = QLineEdit(self.gbTrigNETinfo)
        self.txtTrigNETdaqId.setObjectName(u"txtTrigNETdaqId")
        self.txtTrigNETdaqId.setGeometry(QRect(10, 40, 31, 25))
        self.txtTrigNETmodNo = QLineEdit(self.gbTrigNETinfo)
        self.txtTrigNETmodNo.setObjectName(u"txtTrigNETmodNo")
        self.txtTrigNETmodNo.setGeometry(QRect(70, 40, 31, 25))
        self.cbTrigNETinfoIO = QComboBox(self.gbTrigNETinfo)
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.addItem("")
        self.cbTrigNETinfoIO.setObjectName(u"cbTrigNETinfoIO")
        self.cbTrigNETinfoIO.setGeometry(QRect(120, 40, 81, 25))
        self.label_17 = QLabel(self.gbTrigNETinfo)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setGeometry(QRect(10, 20, 61, 17))
        self.label_18 = QLabel(self.gbTrigNETinfo)
        self.label_18.setObjectName(u"label_18")
        self.label_18.setGeometry(QRect(60, 20, 61, 17))
        self.label_19 = QLabel(self.gbTrigNETinfo)
        self.label_19.setObjectName(u"label_19")
        self.label_19.setGeometry(QRect(130, 20, 67, 17))
        self.groupBox_6 = QGroupBox(self.gbContRotMethod)
        self.groupBox_6.setObjectName(u"groupBox_6")
        self.groupBox_6.setGeometry(QRect(10, 320, 211, 51))
        self.txtAngPerEncode = QLineEdit(self.groupBox_6)
        self.txtAngPerEncode.setObjectName(u"txtAngPerEncode")
        self.txtAngPerEncode.setGeometry(QRect(90, 25, 113, 25))
        self.cbAngPerEncode = QComboBox(self.groupBox_6)
        self.cbAngPerEncode.addItem("")
        self.cbAngPerEncode.addItem("")
        self.cbAngPerEncode.addItem("")
        self.cbAngPerEncode.setObjectName(u"cbAngPerEncode")
        self.cbAngPerEncode.setGeometry(QRect(0, 25, 86, 25))
        self.gbStepByStepMethod = QGroupBox(self.centralwidget)
        self.gbStepByStepMethod.setObjectName(u"gbStepByStepMethod")
        self.gbStepByStepMethod.setGeometry(QRect(260, 330, 251, 411))
        self.btLoadRunInfoFile = QPushButton(self.gbStepByStepMethod)
        self.btLoadRunInfoFile.setObjectName(u"btLoadRunInfoFile")
        self.btLoadRunInfoFile.setGeometry(QRect(10, 30, 121, 27))
        self.btClearRunInfo = QPushButton(self.gbStepByStepMethod)
        self.btClearRunInfo.setObjectName(u"btClearRunInfo")
        self.btClearRunInfo.setGeometry(QRect(180, 30, 61, 27))
        self.label_12 = QLabel(self.gbStepByStepMethod)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(10, 290, 111, 17))
        self.txtOrgAngle = QLineEdit(self.gbStepByStepMethod)
        self.txtOrgAngle.setObjectName(u"txtOrgAngle")
        self.txtOrgAngle.setGeometry(QRect(110, 290, 75, 20))
        font = QFont()
        font.setPointSize(10)
        self.txtOrgAngle.setFont(font)
        self.btReCalc = QPushButton(self.gbStepByStepMethod)
        self.btReCalc.setObjectName(u"btReCalc")
        self.btReCalc.setGeometry(QRect(195, 285, 50, 25))
        self.btReCalc.setFont(font)
        self.label_13 = QLabel(self.gbStepByStepMethod)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(20, 330, 68, 17))
        self.label_14 = QLabel(self.gbStepByStepMethod)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(100, 330, 41, 20))
        self.label_15 = QLabel(self.gbStepByStepMethod)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(170, 330, 91, 17))
        self.txtSbSRunNo = QLineEdit(self.gbStepByStepMethod)
        self.txtSbSRunNo.setObjectName(u"txtSbSRunNo")
        self.txtSbSRunNo.setGeometry(QRect(10, 350, 71, 25))
        self.txtSbSAngle = QLineEdit(self.gbStepByStepMethod)
        self.txtSbSAngle.setObjectName(u"txtSbSAngle")
        self.txtSbSAngle.setGeometry(QRect(90, 350, 71, 25))
        self.txtSbSProtons = QLineEdit(self.gbStepByStepMethod)
        self.txtSbSProtons.setObjectName(u"txtSbSProtons")
        self.txtSbSProtons.setGeometry(QRect(170, 350, 71, 25))
        self.btSbSAdd = QPushButton(self.gbStepByStepMethod)
        self.btSbSAdd.setObjectName(u"btSbSAdd")
        self.btSbSAdd.setGeometry(QRect(170, 380, 71, 27))
        self.lstRunStepInfo = QTreeWidget(self.gbStepByStepMethod)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"1");
        self.lstRunStepInfo.setHeaderItem(__qtreewidgetitem)
        self.lstRunStepInfo.setObjectName(u"lstRunStepInfo")
        self.lstRunStepInfo.setGeometry(QRect(10, 60, 231, 221))
        self.btDoReduction = QPushButton(self.centralwidget)
        self.btDoReduction.setObjectName(u"btDoReduction")
        self.btDoReduction.setGeometry(QRect(260, 750, 161, 31))
        self.groupBox_5 = QGroupBox(self.centralwidget)
        self.groupBox_5.setObjectName(u"groupBox_5")
        self.groupBox_5.setGeometry(QRect(10, 160, 501, 91))
        self.txtNormFactor = QLineEdit(self.groupBox_5)
        self.txtNormFactor.setObjectName(u"txtNormFactor")
        self.txtNormFactor.setGeometry(QRect(110, 25, 113, 25))
        self.label_20 = QLabel(self.groupBox_5)
        self.label_20.setObjectName(u"label_20")
        self.label_20.setGeometry(QRect(10, 30, 111, 17))
        self.label_21 = QLabel(self.groupBox_5)
        self.label_21.setObjectName(u"label_21")
        self.label_21.setGeometry(QRect(240, 30, 121, 17))
        self.txtTimeIndepBack = QLineEdit(self.groupBox_5)
        self.txtTimeIndepBack.setObjectName(u"txtTimeIndepBack")
        self.txtTimeIndepBack.setGeometry(QRect(370, 25, 113, 25))
        self.label_22 = QLabel(self.groupBox_5)
        self.label_22.setObjectName(u"label_22")
        self.label_22.setGeometry(QRect(10, 60, 101, 17))
        self.txtDetEffi = QLineEdit(self.groupBox_5)
        self.txtDetEffi.setObjectName(u"txtDetEffi")
        self.txtDetEffi.setGeometry(QRect(80, 60, 301, 20))
        DataReductionPanel.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(DataReductionPanel)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 522, 22))
        DataReductionPanel.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(DataReductionPanel)
        self.statusbar.setObjectName(u"statusbar")
        DataReductionPanel.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.rbContRotMethod, self.rbStepByStepMethod)
        QWidget.setTabOrder(self.rbStepByStepMethod, self.txtEi)
        QWidget.setTabOrder(self.txtEi, self.txtEnergyBin)
        QWidget.setTabOrder(self.txtEnergyBin, self.txtEnergyMin)
        QWidget.setTabOrder(self.txtEnergyMin, self.txtEnergyMax)
        QWidget.setTabOrder(self.txtEnergyMax, self.txMaskFilePath)
        QWidget.setTabOrder(self.txMaskFilePath, self.btSelectMaskFile)
        QWidget.setTabOrder(self.btSelectMaskFile, self.rbUseDRDefault)
        QWidget.setTabOrder(self.rbUseDRDefault, self.rbUseDRScript)
        QWidget.setTabOrder(self.rbUseDRScript, self.btMakeDRTemp)
        QWidget.setTabOrder(self.btMakeDRTemp, self.btSelectDRScript)
        QWidget.setTabOrder(self.btSelectDRScript, self.txtRunNo)
        QWidget.setTabOrder(self.txtRunNo, self.txtStartDeg)
        QWidget.setTabOrder(self.txtStartDeg, self.txtMinDeg)
        QWidget.setTabOrder(self.txtMinDeg, self.txtMaxDeg)
        QWidget.setTabOrder(self.txtMaxDeg, self.txtStepDeg)
        QWidget.setTabOrder(self.txtStepDeg, self.txtTimeSliceStart)
        QWidget.setTabOrder(self.txtTimeSliceStart, self.txtTimeSliceEnd)
        QWidget.setTabOrder(self.txtTimeSliceEnd, self.txtAngPerEncode)
        QWidget.setTabOrder(self.txtAngPerEncode, self.cbAngPerEncode)
        QWidget.setTabOrder(self.cbAngPerEncode, self.txtTrigNETdaqId)
        QWidget.setTabOrder(self.txtTrigNETdaqId, self.txtTrigNETmodNo)
        QWidget.setTabOrder(self.txtTrigNETmodNo, self.cbTrigNETinfoIO)
        QWidget.setTabOrder(self.cbTrigNETinfoIO, self.btLoadRunInfoFile)
        QWidget.setTabOrder(self.btLoadRunInfoFile, self.btClearRunInfo)
        QWidget.setTabOrder(self.btClearRunInfo, self.lstRunStepInfo)
        QWidget.setTabOrder(self.lstRunStepInfo, self.txtOrgAngle)
        QWidget.setTabOrder(self.txtOrgAngle, self.btReCalc)
        QWidget.setTabOrder(self.btReCalc, self.txtSbSRunNo)
        QWidget.setTabOrder(self.txtSbSRunNo, self.txtSbSAngle)
        QWidget.setTabOrder(self.txtSbSAngle, self.txtSbSProtons)
        QWidget.setTabOrder(self.txtSbSProtons, self.btSbSAdd)
        QWidget.setTabOrder(self.btSbSAdd, self.btDoReduction)
        QWidget.setTabOrder(self.btDoReduction, self.txtNormFactor)
        QWidget.setTabOrder(self.txtNormFactor, self.txtTimeIndepBack)
        QWidget.setTabOrder(self.txtTimeIndepBack, self.txtDetEffi)

        self.retranslateUi(DataReductionPanel)

        QMetaObject.connectSlotsByName(DataReductionPanel)
    # setupUi

    def retranslateUi(self, DataReductionPanel):
        DataReductionPanel.setWindowTitle(QCoreApplication.translate("DataReductionPanel", u"D4Mat2Slicer DataReduction", None))
        self.groupBox.setTitle(QCoreApplication.translate("DataReductionPanel", u"Measurement Method", None))
        self.rbContRotMethod.setText(QCoreApplication.translate("DataReductionPanel", u"Continuous", None))
        self.rbStepByStepMethod.setText(QCoreApplication.translate("DataReductionPanel", u"StepByStep", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("DataReductionPanel", u"Energy Transfer Info", None))
        self.label.setText(QCoreApplication.translate("DataReductionPanel", u"Ei = ", None))
        self.label_2.setText(QCoreApplication.translate("DataReductionPanel", u"hw bin =", None))
        self.label_3.setText(QCoreApplication.translate("DataReductionPanel", u"hw range =", None))
        self.label_4.setText(QCoreApplication.translate("DataReductionPanel", u"-", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("DataReductionPanel", u"Mask File", None))
        self.btSelectMaskFile.setText(QCoreApplication.translate("DataReductionPanel", u"Select", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("DataReductionPanel", u"Data Reduction Script", None))
        self.rbUseDRDefault.setText(QCoreApplication.translate("DataReductionPanel", u"Default", None))
        self.rbUseDRScript.setText(QCoreApplication.translate("DataReductionPanel", u"Use Script", None))
        self.btMakeDRTemp.setText(QCoreApplication.translate("DataReductionPanel", u"Template", None))
        self.btSelectDRScript.setText(QCoreApplication.translate("DataReductionPanel", u"Select", None))
        self.label_5.setText(QCoreApplication.translate("DataReductionPanel", u"Script :", None))
        self.lbDRScriptPath.setText("")
        self.gbContRotMethod.setTitle(QCoreApplication.translate("DataReductionPanel", u"Continuous Rotation Method", None))
        self.groupBox_7.setTitle(QCoreApplication.translate("DataReductionPanel", u"Run Number", None))
        self.label_6.setText(QCoreApplication.translate("DataReductionPanel", u"RunNo=", None))
        self.groupBox_8.setTitle(QCoreApplication.translate("DataReductionPanel", u"Ocillation Info", None))
        self.label_7.setText(QCoreApplication.translate("DataReductionPanel", u"Start Ang =", None))
        self.label_8.setText(QCoreApplication.translate("DataReductionPanel", u"Max Ang =", None))
        self.label_9.setText(QCoreApplication.translate("DataReductionPanel", u"Step Ang =", None))
        self.label_16.setText(QCoreApplication.translate("DataReductionPanel", u"Min Ang =", None))
        self.groupBox_9.setTitle(QCoreApplication.translate("DataReductionPanel", u"Time Slice", None))
        self.label_10.setText(QCoreApplication.translate("DataReductionPanel", u"to", None))
        self.label_11.setText(QCoreApplication.translate("DataReductionPanel", u"[sec]", None))
        self.gbTrigNETinfo.setTitle(QCoreApplication.translate("DataReductionPanel", u"TrigNET Info", None))
        self.cbTrigNETinfoIO.setItemText(0, QCoreApplication.translate("DataReductionPanel", u"DIO1", None))
        self.cbTrigNETinfoIO.setItemText(1, QCoreApplication.translate("DataReductionPanel", u"DIO2", None))
        self.cbTrigNETinfoIO.setItemText(2, QCoreApplication.translate("DataReductionPanel", u"DIO3", None))
        self.cbTrigNETinfoIO.setItemText(3, QCoreApplication.translate("DataReductionPanel", u"DIO4", None))
        self.cbTrigNETinfoIO.setItemText(4, QCoreApplication.translate("DataReductionPanel", u"DIO5", None))
        self.cbTrigNETinfoIO.setItemText(5, QCoreApplication.translate("DataReductionPanel", u"DIO6", None))
        self.cbTrigNETinfoIO.setItemText(6, QCoreApplication.translate("DataReductionPanel", u"DIO7", None))
        self.cbTrigNETinfoIO.setItemText(7, QCoreApplication.translate("DataReductionPanel", u"DIO8", None))

        self.label_17.setText(QCoreApplication.translate("DataReductionPanel", u"DaqID", None))
        self.label_18.setText(QCoreApplication.translate("DataReductionPanel", u"ModNo", None))
        self.label_19.setText(QCoreApplication.translate("DataReductionPanel", u"KickerIO", None))
        self.groupBox_6.setTitle(QCoreApplication.translate("DataReductionPanel", u"Angle/Encoder conv", None))
        self.cbAngPerEncode.setItemText(0, QCoreApplication.translate("DataReductionPanel", u"BL01-TL", None))
        self.cbAngPerEncode.setItemText(1, QCoreApplication.translate("DataReductionPanel", u"BL14-TL", None))
        self.cbAngPerEncode.setItemText(2, QCoreApplication.translate("DataReductionPanel", u"OtherVal", None))

        self.gbStepByStepMethod.setTitle(QCoreApplication.translate("DataReductionPanel", u"Step By Step Method", None))
        self.btLoadRunInfoFile.setText(QCoreApplication.translate("DataReductionPanel", u"Add Run List", None))
        self.btClearRunInfo.setText(QCoreApplication.translate("DataReductionPanel", u"Clear", None))
        self.label_12.setText(QCoreApplication.translate("DataReductionPanel", u"Gonio@Zero :", None))
        self.btReCalc.setText(QCoreApplication.translate("DataReductionPanel", u"Calc", None))
        self.label_13.setText(QCoreApplication.translate("DataReductionPanel", u"RunNo", None))
        self.label_14.setText(QCoreApplication.translate("DataReductionPanel", u"Gonio", None))
        self.label_15.setText(QCoreApplication.translate("DataReductionPanel", u"Protons[TP]", None))
        self.btSbSAdd.setText(QCoreApplication.translate("DataReductionPanel", u"Add", None))
        self.btDoReduction.setText(QCoreApplication.translate("DataReductionPanel", u"Set Parameters", None))
        self.groupBox_5.setTitle(QCoreApplication.translate("DataReductionPanel", u"Other params", None))
        self.label_20.setText(QCoreApplication.translate("DataReductionPanel", u"NormFactor = ", None))
        self.label_21.setText(QCoreApplication.translate("DataReductionPanel", u"TimeIndepBack =", None))
        self.label_22.setText(QCoreApplication.translate("DataReductionPanel", u"DetEffi = ", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'D4mat2SlicerQ_XtalParam.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QDialog, QGroupBox,
    QLabel, QPushButton, QRadioButton, QSizePolicy,
    QWidget)

class Ui_XtalParamDialog(object):
    def setupUi(self, XtalParamDialog):
        if not XtalParamDialog.objectName():
            XtalParamDialog.setObjectName(u"XtalParamDialog")
        XtalParamDialog.resize(404, 213)
        self.rd_AllParams = QRadioButton(XtalParamDialog)
        self.rd_AllParams.setObjectName(u"rd_AllParams")
        self.rd_AllParams.setGeometry(QRect(20, 30, 61, 23))
        self.rd_AllParams.setChecked(True)
        self.rd_PartialParam = QRadioButton(XtalParamDialog)
        self.rd_PartialParam.setObjectName(u"rd_PartialParam")
        self.rd_PartialParam.setGeometry(QRect(20, 60, 81, 23))
        self.btLoadXml = QPushButton(XtalParamDialog)
        self.btLoadXml.setObjectName(u"btLoadXml")
        self.btLoadXml.setGeometry(QRect(300, 110, 89, 25))
        self.btSaveXml = QPushButton(XtalParamDialog)
        self.btSaveXml.setObjectName(u"btSaveXml")
        self.btSaveXml.setGeometry(QRect(300, 170, 89, 25))
        self.label = QLabel(XtalParamDialog)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(20, 10, 361, 17))
        self.groupBox = QGroupBox(XtalParamDialog)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(100, 40, 171, 161))
        self.cb_LC = QCheckBox(self.groupBox)
        self.cb_LC.setObjectName(u"cb_LC")
        self.cb_LC.setGeometry(QRect(10, 30, 141, 23))
        self.cb_OR = QCheckBox(self.groupBox)
        self.cb_OR.setObjectName(u"cb_OR")
        self.cb_OR.setGeometry(QRect(10, 50, 151, 23))
        self.cb_PR = QCheckBox(self.groupBox)
        self.cb_PR.setObjectName(u"cb_PR")
        self.cb_PR.setGeometry(QRect(10, 70, 151, 23))
        self.cb_SL = QCheckBox(self.groupBox)
        self.cb_SL.setObjectName(u"cb_SL")
        self.cb_SL.setGeometry(QRect(10, 90, 92, 23))
        self.cb_DF = QCheckBox(self.groupBox)
        self.cb_DF.setObjectName(u"cb_DF")
        self.cb_DF.setGeometry(QRect(10, 110, 151, 23))
        self.cb_DR = QCheckBox(self.groupBox)
        self.cb_DR.setObjectName(u"cb_DR")
        self.cb_DR.setGeometry(QRect(10, 130, 141, 23))

        self.retranslateUi(XtalParamDialog)

        QMetaObject.connectSlotsByName(XtalParamDialog)
    # setupUi

    def retranslateUi(self, XtalParamDialog):
        XtalParamDialog.setWindowTitle(QCoreApplication.translate("XtalParamDialog", u"XtalParam Dialog", None))
        self.rd_AllParams.setText(QCoreApplication.translate("XtalParamDialog", u"All", None))
        self.rd_PartialParam.setText(QCoreApplication.translate("XtalParamDialog", u"Partial", None))
        self.btLoadXml.setText(QCoreApplication.translate("XtalParamDialog", u"Load", None))
        self.btSaveXml.setText(QCoreApplication.translate("XtalParamDialog", u"Save", None))
        self.label.setText(QCoreApplication.translate("XtalParamDialog", u"Save / Load the partial information of XtalParams", None))
        self.groupBox.setTitle(QCoreApplication.translate("XtalParamDialog", u"Information", None))
        self.cb_LC.setText(QCoreApplication.translate("XtalParamDialog", u"Lattice Constants", None))
        self.cb_OR.setText(QCoreApplication.translate("XtalParamDialog", u"Orientation", None))
        self.cb_PR.setText(QCoreApplication.translate("XtalParamDialog", u"Projection", None))
        self.cb_SL.setText(QCoreApplication.translate("XtalParamDialog", u"Slicing", None))
        self.cb_DF.setText(QCoreApplication.translate("XtalParamDialog", u"Diagonal folding", None))
        self.cb_DR.setText(QCoreApplication.translate("XtalParamDialog", u"Data Reduction", None))
    # retranslateUi




#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
FlexCalc : S(Q,hw)に対して、Q,hwを使用した計算を行う
"""
from __future__ import print_function

import Manyo
import Manyo.Utsusemi as mu
import os
import sys
import time
import numpy
import importlib


class FlexCalc(object):
    ##################################################
    def __init__(self, DAT, EX1="I", EX2="Err", PATH=""):
        """
        Constructor

        @param DAT   (ElementContainerMatrix)
        @param EX1   (String) Intensity計算式
        @param EX2   (String) Error計算式
        @param PATH  (String) 計算用関数ファイルの保存場所
        """
        self.CommentHead = "FlexCalc >>> "

        # DATはElementContainerMatrixか
        # if type(DAT)!=type(Manyo.ElementContainerMatrix()):
        if not isinstance(DAT, Manyo.ElementContainerMatrix):
            msg = self.CommentHead + "Error: DAT is not ElementContainerMatrix."
            mu.UtsusemiError(msg)
            raise UserWarning("Invalid Arguments")
        """
        PROCESS = DAT.PutHeader().PutStringVector("DATAPROCESSED")
        if not "TOF TO ENERGY CONVERSION" in PROCESS:
            print self.CommentHead+"Error: You must execute TofToEnergyTransfer function on this data"
            raise "InvalidArgument"
        """
        CDP = mu.UtsusemiCheckDataProcess()
        if not CDP.CheckProcess(DAT, "TOF TO ENERGY", False):
            mu.UtsusemiError(
                self.CommentHead + "Error: You must execute TofToEnergyTransfer function on this data")
            raise UserWarning("Invalid Arguments")

        self.DAT = DAT

        # DATのデータの情報取得
        if DAT.PutHeader().CheckKey("SAMPLETYPE") == 1:
            self.SAMPLETYPE = DAT.PutHeader().PutString("SAMPLETYPE")
        else:
            self.SAMPLETYPE = None

        if (PATH != "") and os.path.exists(PATH):
            self.PATH = PATH
        else:
            if "UTSUSEMI_USR_DIR" in os.environ:
                self.PATH = os.path.join(
                    os.environ["UTSUSEMI_USR_DIR"], "ana", "tmp")
            elif "HOME" in os.environ:
                self.PATH = os.path.join(os.environ["HOME"], "ana", "tmp")
            else:
                self.PATH = os.path.join("ana", "tmp")
            if not os.path.exists(self.PATH):
                self.PATH = ""

        self.filename = "TempFlexCalcFile.py"

        # Import the temporal file to calc at first to be reloaded later.
        self.EX1 = "I"
        self.EX2 = "Err"
        try:
            self._MakeTempFile()
        except:
            raise UserWarning("FlexCalc Error")

        sys.path.insert(0, self.PATH)
        try:
            self.TF = __import__(self.filename.split(".")[0])
        except:
            raise UserWarning("FlexFuncImportError: file = {}".format(self.filename))

        # 計算式は入っているか
        if (EX1 != "I") and (not isinstance(EX1, str)):
            raise UserWarning("Error: EX1 is not String.")

        if (EX2 != "Err") and (not isinstance(EX2, str)):
            raise UserWarning("Error: EX2 is not String.")

        # 計算式や関数ファイルの名前の登録
        self.EX1 = EX1
        self.EX2 = EX2

        # 計算式が空でなければ関数ファイル作成
        if self.EX1 != "I":
            self._MakeTempFile()

    def _MakeTempFile(self):
        """
        関数ファイルの作成
        これを読み込んで実際の計算に用いる。
        @retval None
        """
        # データがPowderなら、引数は|Q|、PowderでなければQx,Qy,Qzで関数を作成
        if self.SAMPLETYPE == "Powder":
            mu.UtsusemiMessage(self.CommentHead + "SAMPELTYPE is Powder.")
            SS = "from numpy import *\n"
            SS += "def func(hw,I,Err,Q):\n"
            SS += "    I={}\n".format(self.EX1)
            SS += "    Err={}\n".format(self.EX2)
            SS += "    return (I,Err)\n"
        else:
            mu.UtsusemiMessage(self.CommentHead + "SAMPELTYPE is not Powder.")
            SS = "from numpy import *\n"
            SS += "def func(hw,I,Err,Qx,Qy,Qz):\n"
            SS += "    I={}\n".format(self.EX1)
            SS += "    Err={}\n".format(self.EX2)
            SS += "    return (I,Err)\n"

        # 関数ファイルのオープンと書き込み
        try:
            fi = open(os.path.join(self.PATH, self.filename), "w")
        except PermissionError:
            mu.UtsusemiError(self.CommentHead + "Cannot open file:" + os.path.join(self.PATH, self.filename))
            raise UserWarning("ErrorOnOpenFile")

        fi.write(SS)
        fi.close()

    def DoFuncSelf(self):
        """
        計算実行
        @retval None
        """
        self._dofunc(self.DAT)

    def DoFunc(self):
        """
        オリジナルデータをコピーして計算の実行
        @retval new ElementContainerMatrix
        """
        DAT = Manyo.ElementContainerMatrix(self.DAT)
        self._dofunc(DAT)
        return DAT

    def _dofunc(self, DAT):
        """
        計算実行
        関数ファイルを読み込み関数登録して、実行する
        @param DAT (ElementContainerMatrix)
        @retval None
        """

        # 関数ファイルの再インポート
        try:
            self.TF = importlib.reload(self.TF)
        except:
            raise UserWarning("FlexFuncImportError: failed to reload")

        # 入射エネルギーの入手とki算出
        Ei = self.DAT.PutHeader().PutDouble("Ei")
        Ki = numpy.sqrt(Ei / 2.072)

        # 関数の実行
        if self.SAMPLETYPE == "Powder":
            for i in range(DAT.PutTableSize()):
                eca = DAT(i)
                mu.UtsusemiMessage(self.CommentHead + "PSD no={}".format(i))
                for j in range(eca.PutTableSize()):
                    pangle_vec = eca(j).PutHeader(
                    ).PutDoubleVector("PixelPolarAngle")

                    xx_org = eca(j).PutXList()
                    xx = []
                    for k in range(len(xx_org) - 1):
                        xx.append((xx_org[k] + xx_org[k + 1]) / 2.0)
                    xx = numpy.array(xx)
                    yy = numpy.array(eca(j).PutYList())
                    ee = numpy.array(eca(j).PutEList())
                    qq = self._calcQarray(Ei, Ki, pangle_vec[0], xx)
                    xkey = eca(j).PutXKey()
                    ykey = eca(j).PutYKey()
                    ekey = eca(j).PutEKey()

                    (res_yy, res_ee) = self.TF.func(xx, yy, ee, qq)
                    eca(j).Replace(ykey, res_yy.tolist())
                    eca(j).Replace(ekey, res_ee.tolist())
                    eca(j).SetKeys(xkey, ykey, ekey)
            # print xx
            # print qq
        else:
            for i in range(DAT.PutTableSize()):
                eca = DAT(i)
                mu.UtsusemiMessage(self.CommentHead + "PSD no={}".format(i))
                for j in range(eca.PutTableSize()):
                    pangle_vec = eca(j).PutHeader(
                    ).PutDoubleVector("PixelPolarAngle")
                    aangle_vec = eca(j).PutHeader(
                    ).PutDoubleVector("PixelAzimAngle")
                    xx_org = eca(j).PutXList()
                    xx = []
                    for k in range(len(xx_org) - 1):
                        xx.append((xx_org[k] + xx_org[k + 1]) / 2.0)
                    xx = numpy.array(xx)
                    yy = numpy.array(eca(j).PutYList())
                    ee = numpy.array(eca(j).PutEList())
                    (qx, qy, qz) = self._calcQxyzarray(
                        Ei, Ki, pangle_vec[0], aangle_vec[0], xx)
                    xkey = eca(j).PutXKey()
                    ykey = eca(j).PutYKey()
                    ekey = eca(j).PutEKey()

                    (res_yy, res_ee) = self.TF.func(xx, yy, ee, qx, qy, qz)
                    eca(j).Replace(ykey, res_yy.tolist())
                    eca(j).Replace(ekey, res_ee.tolist())
                    eca(j).SetKeys(xkey, ykey, ekey)

    def _calcQarray(self, Ei, Ki, pangle, hw):
        """
        |Q|の計算
        @param Ei     (Double) 入射エネルギー
        @param Ki     (Double) 入射波数ベクトル
        @param pangle (Double) PixelのPolar Angle
        @param hw     (numpy.array) エネルギー遷移情報
        @retval numpy.array of MomentTransfer
        """
        Kf = numpy.sqrt((Ei - hw) / 2.072)
        return numpy.sqrt(Ki * Ki + Kf * Kf - 2. * Ki * Kf * numpy.cos(pangle * numpy.pi / 180.0))

    def _calcQxyzarray(self, Ei, Ki, pangle, aangle, hw):
        """
        (Qx, Qy, Qz)の計算
        @param Ei     (Double) 入射エネルギー
        @param Ki     (Double) 入射波数ベクトル
        @param pangle (Double) PixelのPolar Angle
        @param aangle (Double) Azimuthal Angle
        @param hw     (numpy.array) エネルギー遷移情報
        @retval tupples of numpy.array of MomentTransfer
        """
        Kf = numpy.sqrt((Ei - hw) / 2.072)
        qx = Kf * numpy.sin(pangle * numpy.pi / 180.0) * numpy.cos(aangle * numpy.pi / 180.0)
        qy = Kf * numpy.sin(pangle * numpy.pi / 180.0) * numpy.sin(aangle * numpy.pi / 180.0)
        qz = Kf * numpy.cos(pangle * numpy.pi / 180.0)
        return (qx, qy, qz)

    def finish(self):
        """
        終了処理
        """
        if sys.platform == 'linux':
            com = "rm -rf " + os.path.join(self.PATH, self.filename)
            os.system(com)
